@@ -71,7 +71,8 @@ two_way_table_new (GHashFunc    hash_func,
 	TwoWayTable *table = g_new (TwoWayTable, 1);
 
 	g_return_val_if_fail (base >= 0, NULL);
-	table->key_to_idx  = g_hash_table_new (hash_func, key_compare_func);
+	table->all_keys    = g_hash_table_new (g_direct_hash, g_direct_equal);
+	table->unique_keys = g_hash_table_new (hash_func, key_compare_func);
 	table->idx_to_key  = g_ptr_array_new ();
 	table->base        = base;
 
@@ -87,7 +88,8 @@ two_way_table_new (GHashFunc    hash_func,
 void
 two_way_table_free (TwoWayTable *table)
 {
-	g_hash_table_destroy (table->key_to_idx);
+	g_hash_table_destroy (table->all_keys);
+	g_hash_table_destroy (table->unique_keys);
 	g_ptr_array_free (table->idx_to_key, TRUE);
 	g_free (table);
 }
@@ -96,7 +98,7 @@ two_way_table_free (TwoWayTable *table)
  * two_way_table_put
  * @table  Table
  * @key    Key to enter
- * @unique True if key is entered also if already in table
+ * @potentially_unique True if key is entered also if already in table
  * @apf    Function to call after putting.
  *
  * Puts a key to the TwoWayTable if it is not already there. Returns
@@ -106,18 +108,28 @@ two_way_table_free (TwoWayTable *table)
  */
 gint
 two_way_table_put (const TwoWayTable *table, gpointer key,
-		   gboolean unique,  AfterPutFunc apf, gconstpointer closure)
+		   gboolean potentially_unique,
+		   AfterPutFunc apf, gconstpointer closure)
 {
 	gint index = two_way_table_key_to_idx (table, key);
 	gboolean found = (index >= 0);
-	gboolean addit = !found || !unique;
+	gboolean addit = !found || !potentially_unique;
+	gpointer unique;
 
 	if (addit) {
 		index = table->idx_to_key->len + table->base;
 
-		if (!found)
-			g_hash_table_insert (table->key_to_idx, key,
+		if (!found) {
+			/* We have not seen this pointer before, but is the key
+			 * already in the set ?
+			 */
+			unique = g_hash_table_lookup (table->all_keys, key);
+			if (unique == NULL)
+				g_hash_table_insert (table->all_keys, key,
+						     GINT_TO_POINTER (index + 1));
+			g_hash_table_insert (table->unique_keys, key,
 					     GINT_TO_POINTER (index + 1));
+		}
 		g_ptr_array_add (table->idx_to_key, key);
 	}
 
@@ -140,9 +152,9 @@ two_way_table_replace (const TwoWayTable *table, gint idx, gpointer key)
 {
 	gpointer old_key = two_way_table_idx_to_key (table, idx);
 
-	g_hash_table_remove(table->key_to_idx, old_key);
-	g_hash_table_insert(table->key_to_idx, key, GINT_TO_POINTER (idx + 1));
-	g_ptr_array_index (table->idx_to_key, idx) = key;
+	g_hash_table_remove (table->all_keys, old_key);
+	g_hash_table_insert (table->all_keys, key, GINT_TO_POINTER (idx + 1));
+	g_ptr_array_index   (table->idx_to_key, idx) = key;
 
 	return old_key;
 }
@@ -157,8 +169,7 @@ two_way_table_replace (const TwoWayTable *table, gint idx, gpointer key)
 gint
 two_way_table_key_to_idx (const TwoWayTable *table, gconstpointer key)
 {
-	return GPOINTER_TO_INT (g_hash_table_lookup (table->key_to_idx, key))
-		- 1;
+	return GPOINTER_TO_INT (g_hash_table_lookup (table->unique_keys, key)) - 1;
 }
 
 /**
@@ -315,8 +326,8 @@ destroy_xl_font_widths (void)
 
 
 double
-lookup_font_base_char_width_new (char const * const name, double size_pts,
-				 gboolean const is_default)
+lookup_font_base_char_width (char const *name, double size_pts,
+			     gboolean const is_default)
 {
 	static gboolean need_init = TRUE;
 	gpointer res;
@@ -369,13 +380,13 @@ lookup_font_base_char_width_new (char const * const name, double size_pts,
 
 
 #ifdef HAVE_ICONV
-static char *
+static char const *
 get_locale_charset_name (void)
 {
 	const char *ccharset;
-	static char* charset = NULL;
+	static char const * charset = NULL;
 
-	if (charset)
+	if (charset != NULL)
 		return charset;
 
 #ifdef _NL_CTYPE_CODESET_NAME
@@ -384,16 +395,15 @@ get_locale_charset_name (void)
 	ccharset = nl_langinfo (CODESET);
 #else
 	{
-		const char *locale = setlocale (LC_CTYPE, NULL);
-		const char *tmp = strchr (locale, '.');
-		if (tmp)
-			ccharset = tmp + 1;
+		char const *locale = setlocale (LC_CTYPE, NULL);
+		if (locale != NULL) {
+			const char *tmp = strchr (locale, '.');
+			if (tmp != NULL)
+				ccharset = tmp + 1;
+		}
 	}
 #endif
-	if (!ccharset)
-		ccharset = "ISO-8859-1";
-	charset = g_strdup (ccharset);
-	return charset;
+	return (charset = (charset != NULL) ? g_strdup (charset) : "ISO-8859-1");
 }
 #endif
 
@@ -406,12 +416,40 @@ typedef struct
 /* here is a list of languages for which cp1251 is used on Windows*/
 static char const * const cyr_locales[] =
 {
-	"russian", "ru", "be", "uk", "ukrainian", NULL
+	"be", "be_BY", "bulgarian", "bg", "bg_BG", "mk", "mk_MK",
+	"russian", "ru", "ru_RU", "ru_UA", "sp", "sp_YU", "sr", "sr_YU",
+	"ukrainian", "uk", "uk_UA", NULL
 };
 
+
+/* here is a list of languages for which cp for cjk is used on Windows*/
+static char const * const jp_locales[] =
+{
+	"japan", "japanese", "ja", "ja_JP", NULL
+};
+
+static char const * const zhs_locales[] =
+{
+	"chinese-s", "zh", "zh_CN", NULL
+};
+
+static char const * const kr_locales[] =
+{
+	"korean", "ko", "ko_KR", NULL
+};
+
+static char const * const zht_locales[] =
+{
+	"chinese-t", "zh_HK", "zh_TW", NULL
+ };
+ 
 static s_hash_entry const win_codepages[]=
 {
 	{ cyr_locales , 1251 },
+	{ jp_locales  , 932 },
+	{ zhs_locales , 936 },
+	{ kr_locales  , 949 },
+	{ zht_locales , 950 },
 	{ NULL } /*terminator*/
 };
 
@@ -425,11 +463,13 @@ excel_iconv_win_codepage (void)
 
 		if ((lang = getenv("WINDOWS_LANGUAGE")) == NULL) {
 			char const *locale = setlocale (LC_CTYPE, NULL);
-			char const *lang_sep = strchr (locale, '_');
-			if (lang_sep)
-				lang = g_strndup (locale, lang_sep - locale);
-			else
-				lang = g_strdup (locale); /* simplifies exit */
+			if (locale != NULL) {
+				char const *lang_sep = strchr (locale, '.');
+				if (lang_sep)
+				    lang = g_strndup (locale, lang_sep - locale);
+				else
+				    lang = g_strdup (locale); /* simplifies exit */
+			}
 		}
 
 		if (lang != NULL) {
