@@ -16,7 +16,8 @@
 typedef struct _TwoWayTable   TwoWayTable;
 
 struct _TwoWayTable {
-	GHashTable *key_to_idx;
+	GHashTable *all_keys;
+	GHashTable *unique_keys;
 	GPtrArray  *idx_to_key;
 	gint       base;	/* Indices assigned consecutively from base */
 };
@@ -24,7 +25,7 @@ struct _TwoWayTable {
 typedef void (*AfterPutFunc) (gconstpointer key,
 			      gboolean      was_added,
 			      gint          index,
-			      gpointer      closure);
+			      gconstpointer closure);
 
 TwoWayTable *
 two_way_table_new (GHashFunc    hash_func,
@@ -36,7 +37,7 @@ two_way_table_free (TwoWayTable *table);
 
 gint
 two_way_table_put (const TwoWayTable *table, gpointer key,
-		   gboolean unique,  AfterPutFunc apf, gpointer closure);
+		   gboolean unique,  AfterPutFunc apf, gconstpointer closure);
 
 gpointer
 two_way_table_replace (const TwoWayTable *table, gint idx, gpointer key);
@@ -49,9 +50,8 @@ two_way_table_idx_to_key (const TwoWayTable *table, gint idx);
 
 /* Measures base character width for column sizing. Returns width. */
 /* A new version based on hard coded tables to match XL */
-double
-lookup_font_base_char_width_new (char const * const name, double size_pts,
-				 gboolean const is_default);
+double lookup_font_base_char_width (char const *name, double size_pts,
+				    gboolean const is_default);
 
 
 /* a group of iconv_* - like functions, with safe fallbacks if iconv is
