@@ -91,7 +91,8 @@ val_to_base (FunctionEvalInfo *ei, Value **argv, int num_argv,
 		if (v == 0)
 			max = 1;
 		else
-			max = (int)(loggnum (v + 0.5) / loggnum (dest_base)) + 1;
+			max = (int)(loggnum (v + 0.5) /
+				    loggnum (dest_base)) + 1;
 	}
 
 	if (places > max)
@@ -102,7 +103,8 @@ val_to_base (FunctionEvalInfo *ei, Value **argv, int num_argv,
 	for (digit = max - 1; digit >= 0; digit--) {
 		int thisdigit = fmod (v + 0.5, dest_base);
 		v = floor ((v + 0.5) / dest_base);
-		buffer[digit] = thisdigit["0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"];
+		buffer[digit] = 
+			thisdigit["0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"];
 	}
 	buffer[max] = 0;
 	return value_new_string (buffer);
@@ -116,8 +118,8 @@ static const char *help_bin2dec = {
 
 	   "@DESCRIPTION="
 	   "BIN2DEC function converts a binary number "
-	   "in string or number to its decimal equivalent.\n"
-	   "This function is Excel compatible. "
+	   "in string or number to its decimal equivalent.\n\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "BIN2DEC(101) equals 5.\n"
@@ -140,10 +142,10 @@ static const char *help_bin2oct = {
 	   "@DESCRIPTION="
 	   "BIN2OCT function converts a binary number to an octal number. "
 	   "@places is an optional field, specifying to zero pad to that "
-	   "number of spaces."
+	   "number of spaces.\n"
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If @places is too small or negative #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "BIN2OCT(110111) equals 67.\n"
@@ -166,10 +168,10 @@ static const char *help_bin2hex = {
 	   "@DESCRIPTION="
 	   "BIN2HEX function converts a binary number to a "
 	   "hexadecimal number.  @places is an optional field, specifying "
-	   "to zero pad to that number of spaces."
+	   "to zero pad to that number of spaces.\n"
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If @places is too small or negative #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "BIN2HEX(100111) equals 27.\n"
@@ -192,10 +194,10 @@ static const char *help_dec2bin = {
 	   "@DESCRIPTION="
 	   "DEC2BIN function converts a decimal number to a binary number. "
 	   "@places is an optional field, specifying to zero pad to that "
-	   "number of spaces."
+	   "number of spaces.\n"
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If @places is too small or negative #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "DEC2BIN(42) equals 101010.\n"
@@ -218,10 +220,10 @@ static const char *help_dec2oct = {
 	   "@DESCRIPTION="
 	   "DEC2OCT function converts a decimal number to an octal number. "
 	   "@places is an optional field, specifying to zero pad to that "
-	   "number of spaces."
+	   "number of spaces.\n"
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If @places is too small or negative #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "DEC2OCT(42) equals 52.\n"
@@ -244,10 +246,10 @@ static const char *help_dec2hex = {
 	   "@DESCRIPTION="
 	   "DEC2HEX function converts a decimal number to a hexadecimal "
 	   "number. @places is an optional field, specifying to zero pad "
-	   "to that number of spaces."
+	   "to that number of spaces.\n"
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If @places is too small or negative #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "DEC2HEX(42) equals 2A.\n"
@@ -269,8 +271,8 @@ static const char *help_oct2dec = {
 
 	   "@DESCRIPTION="
 	   "OCT2DEC function converts an octal number "
-	   "in a string or number to its decimal equivalent.\n"
-	   "This function is Excel compatible. "
+	   "in a string or number to its decimal equivalent.\n\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "OCT2DEC(\"124\") equals 84.\n"
@@ -294,9 +296,9 @@ static const char *help_oct2bin = {
 	   "OCT2BIN function converts an octal number to a binary "
 	   "number.  @places is an optional field, specifying to zero pad "
 	   "to that number of spaces.\n"
-	   "This function is Excel compatible. "
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned."
+	   "* If @places is too small or negative #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "OCT2BIN(\"213\") equals 10001011.\n"
@@ -320,9 +322,9 @@ static const char *help_oct2hex = {
 	   "OCT2HEX function converts an octal number to a hexadecimal "
 	   "number.  @places is an optional field, specifying to zero pad "
 	   "to that number of spaces.\n"
-	   "This function is Excel compatible. "
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned."
+	   "* If @places is too small or negative #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "OCT2HEX(132) equals 5A.\n"
@@ -345,10 +347,10 @@ static const char *help_hex2bin = {
 	   "@DESCRIPTION="
 	   "HEX2BIN function converts a hexadecimal number to a binary "
 	   "number.  @places is an optional field, specifying to zero pad "
-	   "to that number of spaces."
+	   "to that number of spaces.\n"
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If @places is too small or negative #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "HEX2BIN(\"2A\") equals 101010.\n"
@@ -371,10 +373,10 @@ static const char *help_hex2oct = {
 	   "@DESCRIPTION="
 	   "HEX2OCT function converts a hexadecimal number to an octal "
 	   "number.  @places is an optional field, specifying to zero pad "
-	   "to that number of spaces."
+	   "to that number of spaces.\n"
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If @places is too small or negative #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "HEX2OCT(\"2A\") equals 52.\n"
@@ -396,8 +398,8 @@ static const char *help_hex2dec = {
 
 	   "@DESCRIPTION="
 	   "HEX2DEC function converts a hexadecimal number "
-	   "to its decimal equivalent.\n"
-	   "This function is Excel compatible. "
+	   "to its decimal equivalent.\n\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "HEX2DEC(\"2A\") equals 42.\n"
@@ -419,14 +421,14 @@ static const char *help_besseli = {
 
 	   "@DESCRIPTION="
 	   "BESSELI function returns the Neumann, Weber or Bessel "
-	   "function. "
+	   "function.\n\n"
 	   "@x is where the function is evaluated. "
 	   "@y is the order of the bessel function, if non-integer it is "
-	   "truncated."
+	   "truncated.\n"
 	   "\n"
-	   "If @x or @y are not numeric a #VALUE! error is returned. "
-	   "If @y < 0 a #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If @x or @y are not numeric a #VALUE! error is returned.\n"
+	   "* If @y < 0 a #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "BESSELI(0.7,3) equals 0.007367374.\n"
@@ -460,11 +462,11 @@ static const char *help_besselk = {
 	   "function. "
 	   "@x is where the function is evaluated. "
 	   "@y is the order of the bessel function, if non-integer it is "
-	   "truncated."
+	   "truncated.\n"
 	   "\n"
-	   "If x or n are not numeric a #VALUE! error is returned. "
-	   "If y < 0 a #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If x or n are not numeric a #VALUE! error is returned.\n"
+	   "* If y < 0 a #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "BESSELK(3,9) equals 397.95880.\n"
@@ -496,11 +498,11 @@ static const char *help_besselj = {
 	   "BESSELJ function returns the bessel function with "
 	   "@x is where the function is evaluated. "
 	   "@y is the order of the bessel function, if non-integer it is "
-	   "truncated."
+	   "truncated.\n"
 	   "\n"
-	   "If @x or @y are not numeric a #VALUE! error is returned.  "
-	   "If @y < 0 a #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If @x or @y are not numeric a #VALUE! error is returned.\n"
+	   "* If @y < 0 a #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "BESSELJ(0.89,3) equals 0.013974004.\n"
@@ -530,14 +532,14 @@ static const char *help_bessely = {
 
 	   "@DESCRIPTION="
 	   "BESSELY function returns the Neumann, Weber or Bessel "
-	   "function. "
+	   "function.\n\n"
 	   "@x is where the function is evaluated. "
 	   "@y is the order of the bessel function, if non-integer it is "
-	   "truncated."
+	   "truncated.\n"
 	   "\n"
-	   "If x or n are not numeric a #VALUE! error is returned. "
-	   "If n < 0 a #NUM! error is returned.\n"
-	   "This function is Excel compatible. "
+	   "* If x or n are not numeric a #VALUE! error is returned.\n"
+	   "* If n < 0 a #NUM! error is returned.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "BESSELY(4,2) equals 0.215903595.\n"
@@ -571,88 +573,88 @@ static const char *help_convert = {
 	   "another.  For example, you can convert a weight in pounds "
 	   "to a weight in grams.  @number is the value you want to "
 	   "convert, @from_unit specifies the unit of the @number, and "
-	   "@to_unit is the unit for the result."
+	   "@to_unit is the unit for the result.\n"
 	   "\n"
 	   "@from_unit and @to_unit can be any of the following:\n\n"
 	   "Weight and mass:\n"
-	   "'g'    Gram\n"
-           "'sg'   Slug\n"
-	   "'lbm'  Pound\n"
-	   "'u'    U (atomic mass)\n"
-	   "'ozm'  Ounce\n\n"
+	   "\t'g'  \t\tGram\n"
+           "\t'sg' \t\tSlug\n"
+	   "\t'lbm'\t\tPound\n"
+	   "\t'u'  \t\tU (atomic mass)\n"
+	   "\t'ozm'\t\tOunce\n\n"
 	   "Distance:\n"
-	   "'m'    Meter\n"
-	   "'mi'   Statute mile\n"
-	   "'Nmi'  Nautical mile\n"
-	   "'in'   Inch\n"
-	   "'ft'   Foot\n"
-	   "'yd'   Yard\n"
-	   "'ang'  Angstrom\n"
-	   "'Pica' Pica\n\n"
+	   "\t'm'   \t\tMeter\n"
+	   "\t'mi'  \t\tStatute mile\n"
+	   "\t'Nmi' \t\tNautical mile\n"
+	   "\t'in'  \t\tInch\n"
+	   "\t'ft'  \t\tFoot\n"
+	   "\t'yd'  \t\tYard\n"
+	   "\t'ang' \t\tAngstrom\n"
+	   "\t'Pica'\t\tPica\n\n"
 	   "Time:\n"
-	   "'yr'   Year\n"
-	   "'day'  Day\n"
-	   "'hr'   Hour\n"
-	   "'mn'   Minute\n"
-	   "'sec'  Second\n\n"
+	   "\t'yr'  \t\tYear\n"
+	   "\t'day' \t\tDay\n"
+	   "\t'hr'  \t\tHour\n"
+	   "\t'mn'  \t\tMinute\n"
+	   "\t'sec' \t\tSecond\n\n"
 	   "Pressure:\n"
-	   "'Pa'   Pascal\n"
-	   "'atm'  Atmosphere\n"
-	   "'mmHg' mm of Mercury\n\n"
+	   "\t'Pa'  \t\tPascal\n"
+	   "\t'atm' \t\tAtmosphere\n"
+	   "\t'mmHg'\tmm of Mercury\n\n"
 	   "Force:\n"
-	   "'N'    Newton\n"
-	   "'dyn'  Dyne\n"
-	   "'lbf'  Pound force\n\n"
+	   "\t'N'   \t\tNewton\n"
+	   "\t'dyn' \t\tDyne\n"
+	   "\t'lbf' \t\tPound force\n\n"
 	   "Energy:\n"
-	   "'J'    Joule\n"
-	   "'e'    Erg\n"
-	   "'c'    Thermodynamic calorie\n"
-	   "'cal'  IT calorie\n"
-	   "'eV'   Electron volt\n"
-	   "'HPh'  Horsepower-hour\n"
-	   "'Wh'   Watt-hour\n"
-	   "'flb'  Foot-pound\n"
-	   "'BTU'  BTU\n\n"
+	   "\t'J'    \t\tJoule\n"
+	   "\t'e'    \t\tErg\n"
+	   "\t'c'    \t\tThermodynamic calorie\n"
+	   "\t'cal'  \t\tIT calorie\n"
+	   "\t'eV'   \tElectron volt\n"
+	   "\t'HPh'  \tHorsepower-hour\n"
+	   "\t'Wh'   \tWatt-hour\n"
+	   "\t'flb'  \t\tFoot-pound\n"
+	   "\t'BTU'  \tBTU\n\n"
 	   "Power:\n"
-	   "'HP'   Horsepower\n"
-	   "'W'    Watt\n"
+	   "\t'HP'   \tHorsepower\n"
+	   "\t'W'    \tWatt\n\n"
 	   "Magnetism:\n"
-	   "'T'    Tesla\n"
-	   "'ga'   Gauss\n\n"
+	   "\t'T'    \t\tTesla\n"
+	   "\t'ga'   \tGauss\n\n"
 	   "Temperature:\n"
-	   "'C'    Degree Celsius\n"
-	   "'F'    Degree Fahrenheit\n"
-	   "'K'    Degree Kelvin\n\n"
+	   "\t'C'    \t\tDegree Celsius\n"
+	   "\t'F'    \t\tDegree Fahrenheit\n"
+	   "\t'K'    \t\tDegree Kelvin\n\n"
 	   "Liquid measure:\n"
-	   "'tsp'  Teaspoon\n"
-	   "'tbs'  Tablespoon\n"
-	   "'oz'   Fluid ounce\n"
-	   "'cup'  Cup\n"
-	   "'pt'   Pint\n"
-	   "'qt'   Quart\n"
-	   "'gal'  Gallon\n"
-	   "'l'    Liter\n\n"
+	   "\t'tsp'  \t\tTeaspoon\n"
+	   "\t'tbs'  \t\tTablespoon\n"
+	   "\t'oz'   \t\tFluid ounce\n"
+	   "\t'cup'  \tCup\n"
+	   "\t'pt'   \t\tPint\n"
+	   "\t'qt'   \t\tQuart\n"
+	   "\t'gal'  \t\tGallon\n"
+	   "\t'l'    \t\tLiter\n\n"
 	   "For metric units any of the following prefixes can be used:\n"
-	   "'E'  exa    1E+18\n"
-	   "'P'  peta   1E+15\n"
-	   "'T'  tera   1E+12\n"
-	   "'G'  giga   1E+09\n"
-	   "'M'  mega   1E+06\n"
-	   "'k'  kilo   1E+03\n"
-	   "'h'  hecto  1E+02\n"
-	   "'e'  dekao  1E+01\n"
-	   "'d'  deci   1E-01\n"
-	   "'c'  centi  1E-02\n"
-	   "'m'  milli  1E-03\n"
-	   "'u'  micro  1E-06\n"
-	   "'n'  nano   1E-09\n"
-	   "'p'  pico   1E-12\n"
-	   "'f'  femto  1E-15\n"
-	   "'a'  atto   1E-18\n"
+	   "\t'E'  \texa   \t1E+18\n"
+	   "\t'P'  \tpeta  \t1E+15\n"
+	   "\t'T'  \ttera  \t\t1E+12\n"
+	   "\t'G'  \tgiga  \t1E+09\n"
+	   "\t'M'  \tmega  \t1E+06\n"
+	   "\t'k'  \tkilo  \t\t1E+03\n"
+	   "\t'h'  \thecto \t1E+02\n"
+	   "\t'e'  \tdekao \t1E+01\n"
+	   "\t'd'  \tdeci  \t1E-01\n"
+	   "\t'c'  \tcenti \t\t1E-02\n"
+	   "\t'm'  \tmilli \t\t1E-03\n"
+	   "\t'u'  \tmicro \t1E-06\n"
+	   "\t'n'  \tnano  \t1E-09\n"
+	   "\t'p'  \tpico  \t1E-12\n"
+	   "\t'f'  \tfemto \t1E-15\n"
+	   "\t'a'  \tatto  \t\t1E-18\n"
 	   "\n"
-	   "If @from_unit and @to_unit are different types, CONVERT returns "
+	   "* If @from_unit and @to_unit are different types, CONVERT returns "
 	   "#NUM! error.\n"
-	   "This function is Excel compatible."
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "CONVERT(3,\"lbm\",\"g\") equals 1360.7769.\n"
@@ -969,16 +971,17 @@ static const char *help_erf = {
 	   "@SYNTAX=ERF([lower limit,]upper_limit)\n"
 
 	   "@DESCRIPTION="
-	   "With a single argument ERF returns the error function, defined as "
-	   "erf(x) = 2/sqrt(pi)* integral from 0 to x of exp(-t*t) dt. "
+	   "ERF returns the error function.  With a single argument ERF "
+	   "returns the error function, defined as\n\n\t"
+	   "erf(x) = 2/sqrt(pi)* integral from 0 to x of exp(-t*t) dt.\n\n"
 	   "If two arguments are supplied, they are the lower and upper "
-	   "limits of the integral."
+	   "limits of the integral.\n"
 	   "\n"
-	   "If either @lower_limit or @upper_limit is not numeric a "
+	   "* If either @lower_limit or @upper_limit is not numeric a "
 	   "#VALUE! error is returned.\n"
-	   "This function is upward-compatible with that in Excel. "
+	   "* This function is upward-compatible with that in Excel. "
 	   "(If two arguments are supplied, "
-	   "Excel will not allow either to be negative.) "
+	   "Excel will not allow either to be negative.)\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "ERF(0.4) equals 0.428392355.\n"
@@ -1016,11 +1019,11 @@ static const char *help_erfc = {
 
 	   "@DESCRIPTION="
 	   "ERFC function returns the complementary "
-	   "error function, defined as 1 - erf(x). "
+	   "error function, defined as\n\n\t1 - erf(x).\n\n"
 	   "erfc(x) is calculated more accurately than 1 - erf(x) for "
-	   "arguments larger than about 0.5."
+	   "arguments larger than about 0.5.\n"
 	   "\n"
-	   "If @x is not numeric a #VALUE! error is returned.  "
+	   "* If @x is not numeric a #VALUE! error is returned.  "
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "ERFC(6) equals 2.15197367e-17.\n"
@@ -1046,11 +1049,10 @@ static const char *help_delta = {
 
 	   "@DESCRIPTION="
 	   "DELTA function tests for numerical equivalence of two "
-	   "arguments, returning 1 in case of equality.  "
-	   "@y is optional, and defaults to 0."
-	   "\n"
-	   "If either argument is non-numeric returns a #VALUE! error.\n"
-	   "This function is Excel compatible."
+	   "arguments, returning 1 in case of equality.\n\n"
+	   "* @y is optional, and defaults to 0."
+	   "* If either argument is non-numeric returns a #VALUE! error.\n"
+	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "DELTA(42.99,43) equals 0.\n"
@@ -1101,10 +1103,10 @@ static const char *help_gestep = {
 	   "@SYNTAX=GESTEP(x[,y])\n"
 	   "@DESCRIPTION="
 	   "GESTEP function test for if @x is >= @y, returning 1 if it "
-	   "is so, and 0 otherwise. @y is optional, and defaults to 0."
+	   "is so, and 0 otherwise. @y is optional, and defaults to 0.\n"
 	   "\n"
-	   "If either argument is non-numeric returns a #VALUE! error.\n"
-	   "This function is Excel compatible."
+	   "* If either argument is non-numeric returns a #VALUE! error.\n"
+	   "* This function is Excel compatible."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "GESTEP(5,4) equals 1.\n"
