@@ -10,16 +10,20 @@
 #include "gnumeric.h"
 #include "cell.h"
 
+#include "gutils.h"
 #include "workbook.h"
 #include "sheet.h"
 #include "expr.h"
+#include "expr-impl.h"
 #include "rendered-value.h"
 #include "value.h"
 #include "style.h"
 #include "format.h"
 #include "sheet-object-cell-comment.h"
-#include "eval.h"
 #include "sheet-style.h"
+#include "parse-util.h"
+
+#define USE_CELL_POOL 1
 
 /**
  * cell_dirty : Mark the sheet containing the cell as being dirty.
@@ -34,7 +38,7 @@ cell_dirty (Cell *cell)
 
 	/* Cells from the clipboard do not have a sheet attached */
 	if (sheet)
-		sheet_set_dirty(sheet, TRUE);
+		sheet_set_dirty (sheet, TRUE);
 }
 
 /**
@@ -63,7 +67,7 @@ cell_cleanout (Cell *cell)
 		/* Clipboard cells, e.g., are not attached to a sheet.  */
 		if (cell_expr_is_linked (cell))
 			dependent_unlink (CELL_TO_DEP (cell), &cell->pos);
-		expr_tree_unref (cell->base.expression);
+		gnm_expr_unref (cell->base.expression);
 		cell->base.expression = NULL;
 		cell->base.flags &= ~(CELL_HAS_EXPRESSION);
 	}
@@ -76,14 +80,26 @@ cell_cleanout (Cell *cell)
 		rendered_value_destroy (cell->rendered_value);
 		cell->rendered_value = NULL;
 	}
-	if (cell->format) {
-		style_format_unref (cell->format);
-		cell->format = NULL;
-	}
-
-	cell_dirty (cell);
 }
 
+/* The pool from which all cells are allocated.  */
+static gnm_mem_chunk *cell_pool;
+
+/**
+ * cell_new:
+ * @sheet: sheet in which to allocate cell.
+ *
+ * Creates a new cell.
+ */
+Cell *
+cell_new (void)
+{
+	Cell *cell = USE_CELL_POOL ? gnm_mem_chunk_alloc0 (cell_pool) : g_new0 (Cell, 1);
+	cell->base.flags = DEPENDENT_CELL;
+	return cell;
+}
+
+
 /**
  * cell_copy:
  * @cell: existing cell to duplicate
@@ -99,18 +115,18 @@ cell_copy (Cell const *cell)
 
 	g_return_val_if_fail (cell != NULL, NULL);
 
-	new_cell = g_new (Cell, 1);
+	new_cell = cell_new ();
 
 	/* bitmap copy first */
 	*new_cell = *cell;
 
 	/* The new cell is not linked into any of the major management structures */
 	new_cell->base.sheet = NULL;
-	new_cell->base.flags &= ~(DEPENDENT_NEEDS_RECALC|CELL_IN_SHEET_LIST|DEPENDENT_IN_EXPR_LIST);
+	new_cell->base.flags &= ~(DEPENDENT_NEEDS_RECALC|CELL_IN_SHEET_LIST|DEPENDENT_IS_LINKED);
 
 	/* now copy properly the rest */
 	if (cell_has_expr (new_cell))
-		expr_tree_ref (new_cell->base.expression);
+		gnm_expr_ref (new_cell->base.expression);
 
 	new_cell->rendered_value = NULL;
 
@@ -118,9 +134,6 @@ cell_copy (Cell const *cell)
 		? value_duplicate (new_cell->value)
 		: value_new_empty ();
 
-	if (cell->format)
-		style_format_ref (cell->format);
-
 	return new_cell;
 }
 
@@ -136,142 +149,7 @@ cell_destroy (Cell *cell)
 	g_return_if_fail (cell != NULL);
 
 	cell_cleanout (cell);
-	g_free (cell);
-}
-
-/**
- * cell_eval_content:
- * @cell: the cell to evaluate.
- *
- * This function evaluates the contents of the cell,
- * it should not be used by anyone. It is an internal
- * function.
- **/
-gboolean
-cell_eval_content (Cell *cell)
-{
-	static Cell *iterating = NULL;
-	Value   *v;
-	EvalPos	 pos;
-	int	 max_iteration;
-
-	if (!cell_has_expr (cell))
-		return TRUE;
-
-#ifdef DEBUG_EVALUATION
-	{
-		ParsePos pp;
-		char *str = expr_tree_as_string (cell->base.expression,
-			parse_pos_init_cell (&pp, cell));
-		printf ("{\nEvaluating %s: %s;\n", cell_name (cell), str);
-		g_free (str);
-	}
-#endif
-
-	/* This is the bottom of a cycle */
-	if (cell->base.flags & DEPENDENT_BEING_CALCULATED) {
-		if (!cell->base.sheet->workbook->iteration.enabled)
-			return TRUE;
-
-		/* but not the first bottom */
-		if (cell->base.flags & CELL_BEING_ITERATED) {
-#ifdef DEBUG_EVALUATION
-			printf ("}; /* already-iterate (%d) */\n", iterating == NULL);
-#endif
-			return iterating == NULL;
-		}
-
-		/* if we are still marked as iterating then make this the last
-		 * time through.
-		 */
-		if (iterating == cell) {
-#ifdef DEBUG_EVALUATION
-			puts ("}; /* NO-iterate (1) */");
-#endif
-			return TRUE;
-		} else if (iterating == NULL) {
-			cell->base.flags |= CELL_BEING_ITERATED;
-			iterating = cell;
-#ifdef DEBUG_EVALUATION
-			puts ("}; /* START iterate = TRUE (0) */");
-#endif
-			return FALSE;
-		} else {
-#ifdef DEBUG_EVALUATION
-			puts ("}; /* other-iterate (0) */");
-#endif
-			return FALSE;
-		}
-	}
-
-	/* Prepare to calculate */
-	eval_pos_init_cell (&pos, cell);
-	cell->base.flags |= DEPENDENT_BEING_CALCULATED;
-	max_iteration = cell->base.sheet->workbook->iteration.max_number;
-
-iterate :
-	v = expr_eval (cell->base.expression, &pos, EVAL_STRICT);
-	if (v == NULL)
-		v = value_new_error (&pos, "Internal error");
-
-#ifdef DEBUG_EVALUATION
-	{
-		char *valtxt = v
-			? value_get_as_string (v)
-			: g_strdup ("NULL");
-		printf ("Evaluation(%d) %s := %s\n", max_iteration, cell_name (cell), valtxt);
-		g_free (valtxt);
-	}
-#endif
-
-	/* The top of a cycle */
-	if (cell->base.flags & CELL_BEING_ITERATED) {
-		cell->base.flags &= ~CELL_BEING_ITERATED;
-
-		/* We just completed the last iteration, don't change things */
-		if (iterating && max_iteration-- > 0) {
-			/* If we are within bounds make this the last round */
-			if (value_diff (cell->value, v) < cell->base.sheet->workbook->iteration.tolerance)
-				max_iteration = 0;
-			else {
-#ifdef DEBUG_EVALUATION
-				puts ("/* iterate == NULL */");
-#endif
-				iterating = NULL;
-			}
-			value_release (cell->value);
-			cell->value = v;
-			puts ("/* LOOP */");
-#ifdef DEBUG_EVALUATION
-#endif
-			goto iterate;
-		}
-		g_return_val_if_fail (iterating, TRUE);
-		iterating = NULL;
-	} else {
-		/* do not use cell_assign_value unless you pass in the format */
-		if (cell->value != NULL)
-			value_release (cell->value);
-		cell->value = v;
-
-		/* Optimization : Since we don't span calculated cells
-		 * it is ok, to wipe rendered values.  The drawing routine
-		 * will handle it.
-		 */
-		if (cell->rendered_value != NULL) {
-			rendered_value_destroy (cell->rendered_value);
-			cell->rendered_value = NULL;
-		}
-	}
-
-	if (iterating == cell)
-		iterating = NULL;
-
-#ifdef DEBUG_EVALUATION
-	printf ("} (%d)\n", iterating == NULL);
-#endif
-	cell->base.flags &= ~DEPENDENT_BEING_CALCULATED;
-	return iterating == NULL;
+	if (USE_CELL_POOL) gnm_mem_chunk_free (cell_pool, cell); else g_free (cell);
 }
 
 /*
@@ -284,32 +162,28 @@ iterate :
  * Auxiliary items canvas items attached to the cell are moved.
  */
 void
-cell_relocate (Cell *cell, ExprRewriteInfo *rwinfo)
+cell_relocate (Cell *cell, GnmExprRewriteInfo const *rwinfo)
 {
 	g_return_if_fail (cell != NULL);
+	g_return_if_fail (rwinfo != NULL);
 
 	/* 1. Tag the cell as dirty */
 	cell_dirty (cell);
 
-	/* 2. If the cell contains a formula, relocate the formula */
+	/* 2. If the cell contains a expr, relocate it */
 	if (cell_has_expr (cell)) {
-		ExprTree *expr = cell->base.expression;
+		GnmExpr const *expr = gnm_expr_rewrite (cell->base.expression, rwinfo);
 
+#warning make this a precondition
 		if (cell_expr_is_linked (cell))
 			dependent_unlink (CELL_TO_DEP (cell), &cell->pos);
 
 		/* bounds check, and adjust local references from the cell */
-		if (rwinfo != NULL) {
-			expr = expr_rewrite (expr, rwinfo);
-
-			if (expr != NULL) {
-				/* expression was unlinked above */
-				expr_tree_unref (cell->base.expression);
-				cell->base.expression = expr;
-			}
+		if (expr != NULL) {
+			gnm_expr_unref (cell->base.expression);
+			cell->base.expression = expr;
 		}
 
-		/* Relink the expression.  */
 		dependent_link (CELL_TO_DEP (cell), &cell->pos);
 	}
 }
@@ -333,8 +207,7 @@ cell_relocate (Cell *cell, ExprRewriteInfo *rwinfo)
 void
 cell_set_text (Cell *cell, char const *text)
 {
-	StyleFormat *format;
-	ExprTree    *expr;
+	GnmExpr const *expr;
 	Value	    *val;
 	ParsePos     pos;
 
@@ -342,19 +215,17 @@ cell_set_text (Cell *cell, char const *text)
 	g_return_if_fail (text != NULL);
 	g_return_if_fail (!cell_is_partial_array (cell));
 
-	format = parse_text_value_or_expr (parse_pos_init_cell (&pos, cell),
+	parse_text_value_or_expr (parse_pos_init_cell (&pos, cell),
 		text, &val, &expr, mstyle_get_format (cell_get_mstyle (cell)));
 
 	if (val != NULL) {	/* String was a value */
 		cell_cleanout (cell);
 		cell->value = val;
-		/* parse_text already refed the format */
-		cell->format = format;
 		cell_render_value (cell, TRUE);
+		cell_dirty (cell);
 	} else {		/* String was an expression */
-		cell_set_expr (cell, expr, format);
-		expr_tree_unref (expr);
-		style_format_unref (format);
+		cell_set_expr (cell, expr);
+		gnm_expr_unref (expr);
 	}
 }
 
@@ -366,8 +237,6 @@ cell_set_text (Cell *cell, char const *text)
  *
  * The value is rendered but spans are not calculated.
  *
- * If an optional format is supplied it is stored for later use.
- *
  * WARNING : This is an internal routine that does not queue redraws,
  *           does not auto-resize, does not calculate spans, does
  *           not mark anything as dirty.
@@ -375,17 +244,11 @@ cell_set_text (Cell *cell, char const *text)
  * NOTE : This DOES NOT check for array partitioning.
  */
 void
-cell_assign_value (Cell *cell, Value *v, StyleFormat *opt_fmt)
+cell_assign_value (Cell *cell, Value *v)
 {
 	g_return_if_fail (cell);
 	g_return_if_fail (v);
 
-	if (opt_fmt)
-		style_format_ref (opt_fmt);
-	if (cell->format)
-		style_format_unref (cell->format);
-	cell->format = opt_fmt;
-
 	if (cell->value != NULL)
 		value_release (cell->value);
 	cell->value = v;
@@ -399,29 +262,21 @@ cell_assign_value (Cell *cell, Value *v, StyleFormat *opt_fmt)
  * The value is rendered but spans are not calculated, then the rendered string
  * is stored as if that is what the user had entered.
  *
- * If an optional format is supplied it is stored for later use.
- *
  * WARNING : This is an internal routine that does not queue redraws,
  *           does not auto-resize, and does not calculate spans.
  *
  * NOTE : This DOES check for array partitioning.
  */
 void
-cell_set_value (Cell *cell, Value *v, StyleFormat *opt_fmt)
+cell_set_value (Cell *cell, Value *v)
 {
-	g_return_if_fail (cell);
-	g_return_if_fail (v);
+	g_return_if_fail (cell != NULL);
+	g_return_if_fail (v != NULL);
 	g_return_if_fail (!cell_is_partial_array (cell));
 
-	if (opt_fmt)
-		style_format_ref (opt_fmt);
-
 	cell_cleanout (cell);
-
-	/* TODO : It would be nice to standardize on NULL == General */
-	cell->format = (opt_fmt == NULL || style_format_is_general (opt_fmt))
-		? NULL : opt_fmt;
 	cell->value = v;
+	cell_dirty (cell);
 }
 
 /*
@@ -430,8 +285,6 @@ cell_set_value (Cell *cell, Value *v, StyleFormat *opt_fmt)
  *        list.  It marks the sheet as dirty. It is intended for use by import
  *        routines or operations that do bulk assignment.
  *
- * If an optional format is supplied it is stored for later use.
- *
  * WARNING : This is an internal routine that does not queue redraws,
  *           does not auto-resize, does not calculate spans, and does
  *           not render the value.
@@ -439,21 +292,18 @@ cell_set_value (Cell *cell, Value *v, StyleFormat *opt_fmt)
  * NOTE : This DOES check for array partitioning.
  */
 void
-cell_set_expr_and_value (Cell *cell, ExprTree *expr, Value *v,
-			 StyleFormat *opt_fmt, gboolean link_expr)
+cell_set_expr_and_value (Cell *cell, GnmExpr const *expr, Value *v,
+			 gboolean link_expr)
 {
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (expr != NULL);
 	g_return_if_fail (!cell_is_partial_array (cell));
 
 	/* Repeat after me.  Ref before unref. */
-	expr_tree_ref (expr);
-	if (opt_fmt != NULL)
-		style_format_ref (opt_fmt);
-
+	gnm_expr_ref (expr);
 	cell_cleanout (cell);
+	cell_dirty (cell);
 
-	cell->format = opt_fmt;
 	cell->base.expression = expr;
 	cell->base.flags |= CELL_HAS_EXPRESSION;
 	cell->value = v;
@@ -463,9 +313,8 @@ cell_set_expr_and_value (Cell *cell, ExprTree *expr, Value *v,
 
 /**
  * cell_set_expr_internal:
- * @cell: the cell to set the formula to
- * @expr: an expression tree with the formula
- * opt_fmt: an optional format to apply to the cell.
+ * @cell: the cell to set the expr for
+ * @expr: an expression
  *
  * A private internal utility to store an expression.
  * Does NOT
@@ -475,30 +324,25 @@ cell_set_expr_and_value (Cell *cell, ExprTree *expr, Value *v,
  * 	- link the expression into the master list.
  */
 static void
-cell_set_expr_internal (Cell *cell, ExprTree *expr, StyleFormat *opt_fmt)
+cell_set_expr_internal (Cell *cell, GnmExpr const *expr)
 {
-	expr_tree_ref (expr);
-	if (opt_fmt != NULL)
-		style_format_ref (opt_fmt);
+	gnm_expr_ref (expr);
 
 	cell_cleanout (cell);
 
-	cell->format = opt_fmt;
 	cell->base.expression = expr;
 	cell->base.flags |= CELL_HAS_EXPRESSION;
 
 	/* Until the value is recomputed, we put in this value.  */
 	cell->value = value_new_empty ();
+	cell_dirty (cell);
 }
 
 /*
  * cell_set_expr_unsafe : Stores and references the supplied expression.  It
  *         marks the sheet as dirty.  Intented for use by import routines that
- *         do bulk assignment.
- *
- * The cell is NOT marked for recalc.
- *
- * If an optional format is supplied it is stored for later use.
+ *         do bulk assignment.  The resulting cell is NOT linked into the
+ *         dependent list.  Nor marked for recalc.
  *
  * WARNING : This is an internal routine that does not queue redraws,
  *           does not auto-resize, and does not calculate spans.
@@ -506,39 +350,45 @@ cell_set_expr_internal (Cell *cell, ExprTree *expr, StyleFormat *opt_fmt)
  *           using this.
  */
 void
-cell_set_expr_unsafe (Cell *cell, ExprTree *expr, StyleFormat *opt_fmt)
+cell_set_expr_unsafe (Cell *cell, GnmExpr const *expr)
 {
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (expr != NULL);
 
-	cell_set_expr_internal (cell, expr, opt_fmt);
+	cell_set_expr_internal (cell, expr);
+}
+
+/**
+ * cell_set_expr :  Stores and references the supplied expression
+ *         marks the sheet as dirty.  Intented for use by import routines that
+ *         do bulk assignment.  The resulting cell _is_ linked into the
+ *         dependent list, but NOT marked for recalc.
+ *
+ * WARNING : This is an internal routine that does not queue redraws,
+ *           does not auto-resize, and does not calculate spans.
+ *           Be very careful using this.
+ */
+void
+cell_set_expr (Cell *cell, GnmExpr const *expr)
+{
+	g_return_if_fail (!cell_is_partial_array (cell));
+	g_return_if_fail (cell != NULL);
+	g_return_if_fail (expr != NULL);
+
+	cell_set_expr_internal (cell, expr);
 	dependent_link (CELL_TO_DEP (cell), &cell->pos);
 }
 
-/**
- * cell_set_expr : A utility wrapper for cell_set_expr_unsafe.  That adds
- *      checks for array subdivision.
- */
-void
-cell_set_expr (Cell *cell, ExprTree *expr, StyleFormat *opt_fmt)
-{
-	g_return_if_fail (!cell_is_partial_array (cell));
-
-	cell_set_expr_unsafe (cell, expr, opt_fmt);
-}
-
 /**
  * cell_set_array_formula:
- * @sheet:   The sheet to set the formula to.
+ * @sheet:   The sheet to set the expr in.
  * @row_a:   The top row in the destination region.
  * @col_a:   The left column in the destination region.
  * @row_b:   The bottom row in the destination region.
  * @col_b:   The right column in the destination region.
- * @formula: an expression tree with the formula
- * @queue_recalc : A flag that if true indicates that the cells should be
- *                 queued for recalc.
+ * @expr:    an expression
  *
- * Uses cell_set_expr_internal to store the formula as an
+ * Uses cell_set_expr_internal to store the expr as an
  * 'array-formula'.  The supplied expression is wrapped in an array
  * operator for each cell in the range and scheduled for recalc.  The
  * upper left corner is handled as a special case and care is taken to
@@ -554,26 +404,24 @@ cell_set_expr (Cell *cell, ExprTree *expr, StyleFormat *opt_fmt)
 void
 cell_set_array_formula (Sheet *sheet,
 			int col_a, int row_a, int col_b, int row_b,
-			ExprTree *formula)
+			GnmExpr const *expr)
 {
 	int const num_rows = 1 + row_b - row_a;
 	int const num_cols = 1 + col_b - col_a;
 	int x, y;
 	Cell * const corner = sheet_cell_fetch (sheet, col_a, row_a);
-	ExprTree *wrapper;
+	GnmExpr const *wrapper;
 
 	g_return_if_fail (num_cols > 0);
 	g_return_if_fail (num_rows > 0);
-	g_return_if_fail (formula != NULL);
+	g_return_if_fail (expr != NULL);
 	g_return_if_fail (corner != NULL);
 	g_return_if_fail (col_a <= col_b);
 	g_return_if_fail (row_a <= row_b);
 
-	wrapper = expr_tree_new_array (0, 0, num_cols, num_rows);
-	wrapper->array.corner.value = NULL;
-	wrapper->array.corner.expr = formula;
-	cell_set_expr_internal (corner, wrapper, NULL);
-	expr_tree_unref (wrapper);
+	wrapper = gnm_expr_new_array (0, 0, num_cols, num_rows, expr);
+	cell_set_expr_internal (corner, wrapper);
+	gnm_expr_unref (wrapper);
 
 	for (x = 0; x < num_cols; ++x)
 		for (y = 0; y < num_rows; ++y) {
@@ -583,10 +431,10 @@ cell_set_array_formula (Sheet *sheet,
 				continue;
 
 			cell = sheet_cell_fetch (sheet, col_a + x, row_a + y);
-			wrapper = expr_tree_new_array (x, y, num_cols, num_rows);
-			cell_set_expr_internal (cell, wrapper, NULL);
+			wrapper = gnm_expr_new_array (x, y, num_cols, num_rows, NULL);
+			cell_set_expr_internal (cell, wrapper);
 			dependent_link (CELL_TO_DEP (cell), &cell->pos);
-			expr_tree_unref (wrapper);
+			gnm_expr_unref (wrapper);
 		}
 
 	dependent_link (CELL_TO_DEP (corner), &corner->pos);
@@ -630,7 +478,7 @@ cell_is_zero (Cell const *cell)
 	case VALUE_INTEGER : return v->v_int.val == 0;
 	case VALUE_FLOAT :
 	{
-		double const res = v->v_float.val;
+		gnum_float res = v->v_float.val;
 		return (-1e-10 < res && res < 1e-10);
 	}
 
@@ -639,11 +487,11 @@ cell_is_zero (Cell const *cell)
 	}
 }
 
-ExprArray const *
+GnmExprArray const *
 cell_is_array (Cell const *cell)
 {
 	if (cell != NULL && cell_has_expr (cell) &&
-	    cell->base.expression->any.oper == OPER_ARRAY)
+	    cell->base.expression->any.oper == GNM_EXPR_OP_ARRAY)
 		return &cell->base.expression->array;
 	return NULL;
 }
@@ -651,7 +499,7 @@ cell_is_array (Cell const *cell)
 gboolean
 cell_is_partial_array (Cell const *cell)
 {
-	ExprArray const *ref = cell_is_array (cell);
+	GnmExprArray const *ref = cell_is_array (cell);
 	return ref != NULL && (ref->cols > 1 || ref->rows > 1);
 }
 
@@ -695,37 +543,29 @@ cell_get_mstyle (Cell const *cell)
 				cell->pos.row);
 }
 
-char *
+/**
+ * cell_get_format :
+ * @cell :
+ *
+ * Get the display format.  If the assigned format is General,
+ * the format of the value will be used.
+ */
+StyleFormat *
 cell_get_format (Cell const *cell)
 {
-	char   *result = NULL;
-	MStyle *mstyle;
+	StyleFormat *fmt;
 
-	g_return_val_if_fail (cell != NULL, g_strdup ("General"));
+	g_return_val_if_fail (cell != NULL, style_format_general ());
 
-	mstyle = cell_get_mstyle (cell);
+	fmt = mstyle_get_format (cell_get_mstyle (cell));
 
-	if (mstyle_is_element_set (mstyle, MSTYLE_FORMAT)) {
-		StyleFormat const *format = mstyle_get_format (mstyle);
+	g_return_val_if_fail (fmt != NULL, style_format_general ());
 
-		/* FIXME: we really should distinguish between "not assigned"
-		 * and "assigned General".
-		 *
-		 * 8/20/00 JEG : Do we still need this test ?
-		 */
-		if (format) {
-			/* If the format is General it may have been over
-			 * ridden by the format used to parse the input text.
-			 */
-			result = style_format_as_XL (format, FALSE);
-			if (!strcmp (result, "General") != 0 && cell->format != NULL) {
-				g_free (result);
-				result = style_format_as_XL (cell->format, FALSE);
-			}
-		}
-	}
+	if (style_format_is_general (fmt) &&
+	    cell->value != NULL && VALUE_FMT (cell->value))
+		fmt = VALUE_FMT (cell->value);
 
-	return result;
+	return fmt;
 }
 
 /*
@@ -775,7 +615,7 @@ cell_convert_expr_to_value (Cell *cell)
 	if (cell_expr_is_linked (cell))
 		dependent_unlink (CELL_TO_DEP (cell), &cell->pos);
 
-	expr_tree_unref (cell->base.expression);
+	gnm_expr_unref (cell->base.expression);
 	cell->base.expression = NULL;
 	cell->base.flags &= ~CELL_HAS_EXPRESSION;
 
@@ -821,3 +661,34 @@ cell_has_comment (const Cell *cell)
 {
 	return cell_has_comment_pos (cell->base.sheet, &cell->pos);
 }
+
+/****************************************************************************/
+
+void
+cell_init (void)
+{
+#if USE_CELL_POOL
+	cell_pool = gnm_mem_chunk_new ("cell pool",
+				       sizeof (Cell),
+				       128 * 1024 - 128);
+#endif
+}
+
+#if USE_CELL_POOL
+static void
+cb_cell_pool_leak (gpointer data, gpointer user)
+{
+	Cell const *cell = data;
+	fprintf (stderr, "Leaking cell %p at %s\n", cell, cell_pos_name (&cell->pos));
+}
+#endif
+
+void
+cell_shutdown (void)
+{
+#if USE_CELL_POOL
+	gnm_mem_chunk_foreach_leak (cell_pool, cb_cell_pool_leak, NULL);
+	gnm_mem_chunk_destroy (cell_pool, FALSE);
+	cell_pool = NULL;
+#endif
+}
