@@ -16,17 +16,21 @@ typedef enum {
 
 	COLLECT_IGNORE_BLANKS = 0x1000,
 	COLLECT_DATES	      = 0x2000,
+	COLLECT_INFO	      = 0x4000,
+
+	COLLECT_NO_INFO_MASK  = 0x3777
 } CollectFlags;
 
 
 typedef int (*float_range_function_t) (const gnum_float *, int, gnum_float *);
 typedef int (*float_range_function2_t) (const gnum_float *, const gnum_float *, int, gnum_float *);
+typedef int (*string_range_function_t) (GSList *, char**);
 
 gnum_float *collect_floats_value (const Value *val, const EvalPos *ep,
 				  CollectFlags flags,
 				  int *n, Value **error);
 
-Value *float_range_function (ExprList *exprlist, FunctionEvalInfo *ei,
+Value *float_range_function (GnmExprList *exprlist, FunctionEvalInfo *ei,
 			     float_range_function_t func,
 			     CollectFlags flags,
 			     char const *func_error);
@@ -35,5 +39,9 @@ Value *float_range_function2 (Value *val0, Value *val1, FunctionEvalInfo *ei,
 			      float_range_function2_t func,
 			      CollectFlags flags,
 			      char const *func_error);
+Value *string_range_function (GnmExprList *exprlist, FunctionEvalInfo *ei,
+			      string_range_function_t func,
+			      CollectFlags flags,
+			      char const *func_error);
 
 #endif
