@@ -2,6 +2,7 @@
  * consolidate.c : Implementation of the data consolidation feature.
  *
  * Copyright (C) Almer S. Tigelaar <almer@gnome.org>
+ * Copyright (C) Andreas J Guelzow <aguelzow@taliesin.ca>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,19 +18,23 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
- 
-#include <config.h>
+
+#include <gnumeric-config.h>
+#include "gnumeric.h"
+#include "consolidate.h"
+
 #include "cell.h"
-#include "eval.h"
+#include "dependent.h"
+#include "expr.h"
 #include "func.h"
 #include "position.h"
 #include "ranges.h"
 #include "selection.h"
 #include "sheet.h"
+#include "value.h"
 #include "str.h"
 #include "workbook.h"
-
-#include "consolidate.h"
+#include "workbook-control.h"
 
 /**********************************************************************************
  * UTILITY ROUTINES
@@ -55,7 +60,7 @@ get_bounding_box (GSList const *granges, Range *box)
 
 	g_return_if_fail (granges != NULL);
 	g_return_if_fail (box != NULL);
-	
+
 	max_x = max_y = 0;
 	for (l = granges; l != NULL; l = l->next) {
 		GlobalRange *gr = l->data;
@@ -86,14 +91,9 @@ get_bounding_box (GSList const *granges, Range *box)
  * it to a value.
  **/
 static void
-set_cell_expr (Sheet *sheet, int const col, int const row, ExprTree *expr)
+set_cell_expr (Sheet *sheet, int col, int row, GnmExpr const *expr)
 {
-	Cell *cell;
-
-	g_return_if_fail (expr != NULL);
-	
-	cell = sheet_cell_fetch (sheet, col, row);
-	cell_set_expr (cell, expr, NULL);
+	cell_set_expr (sheet_cell_fetch (sheet, col, row), expr);
 }
 
 /**
@@ -106,33 +106,31 @@ set_cell_expr (Sheet *sheet, int const col, int const row, ExprTree *expr)
  * Set the value of a cell.
  **/
 static void
-set_cell_value (Sheet *sheet, int const col, int const row, Value const *value)
+set_cell_value (Sheet *sheet, int col, int row, Value const *value)
 {
 	/* There are cases in which value can be NULL */
-	if (value) {
-		Cell *cell;
-		
-		cell = sheet_cell_fetch (sheet, col, row);
-		cell_set_value (cell, value_duplicate (value), NULL);
-	}
+	if (value != NULL)
+		cell_set_value (sheet_cell_fetch (sheet, col, row),
+				value_duplicate (value));
 }
 
 static void
-redraw_respan_and_select (Sheet *sheet, int start_col, int start_row, int end_col, int end_row,
-			  gboolean values)
+redraw_respan_and_select (Sheet *sheet, CellPos const *start,
+			  int end_col, int end_row,
+			  gboolean values, WorkbookView *wbv)
 {
 	Range r;
 
-	range_init (&r, start_col, start_row, end_col, end_row);
+	range_init (&r, start->col, start->row, end_col, end_row);
 	sheet_range_calc_spans (sheet, &r, SPANCALC_RESIZE | SPANCALC_RENDER);
 	sheet_region_queue_recalc (sheet, &r);
-	
+
 	if (values) {
 		int row, col;
-		
+
 		workbook_recalc (sheet->workbook);
-		for (row = start_row; row <= end_row; row++) {
-			for (col = start_col; col <= end_col; col++) {
+		for (row = start->row; row <= end_row; row++) {
+			for (col = start->col; col <= end_col; col++) {
 				Cell *cell = sheet_cell_fetch (sheet, col, row);
 
 				if (cell_has_expr (cell))
@@ -140,11 +138,11 @@ redraw_respan_and_select (Sheet *sheet, int start_col, int start_row, int end_co
 			}
 		}
 	}
-	
-	sheet_redraw_range (sheet, &r);	
-	sheet_selection_set (sheet, end_col, end_row,
-			     start_col, start_row,
-			     end_col, end_row);	
+
+	sheet_redraw_range (sheet, &r);
+	sv_selection_set (sheet_get_view (sheet, wbv), start,
+			  start->col, start->row,
+			  end_col, end_row);
 }
 
 static int
@@ -170,13 +168,13 @@ Consolidate *
 consolidate_new (void)
 {
 	Consolidate *cs;
-	
+
 	cs = g_new0 (Consolidate, 1);
 	cs->fd = NULL;
 	cs->dst = NULL;
 	cs->src = NULL;
 	cs->mode = CONSOLIDATE_PUT_VALUES;
-	
+
 	return cs;
 }
 
@@ -186,7 +184,7 @@ consolidate_free (Consolidate *cs)
 	GSList *l;
 
 	g_return_if_fail (cs != NULL);
-		
+
 	if (cs->fd) {
 		func_unref (cs->fd);
 		cs->fd = NULL;
@@ -209,7 +207,7 @@ consolidate_set_function (Consolidate *cs, FunctionDefinition *fd)
 {
 	g_return_if_fail (cs != NULL);
 	g_return_if_fail (fd != NULL);
-	
+
 	if (cs->fd)
 		func_unref (cs->fd);
 
@@ -226,17 +224,19 @@ consolidate_set_mode (Consolidate *cs, ConsolidateMode mode)
 }
 
 gboolean
-consolidate_set_destination (Consolidate *cs, Sheet *sheet, Range const *r)
+consolidate_set_destination (Consolidate *cs, Value *range)
 {
 	GlobalRange *new;
 	GSList const *l;
-	
-	g_return_val_if_fail (cs != NULL, FALSE);
-	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
-	g_return_val_if_fail (r != NULL, FALSE);
 
-	new = global_range_new (sheet, r);
-	
+	g_return_val_if_fail (cs != NULL, FALSE);
+	g_return_val_if_fail (range != NULL, FALSE);
+
+	new = g_new (GlobalRange, 1);
+
+	new->sheet = range->v_range.cell.a.sheet;
+	setup_range_from_value (&new->range, range, TRUE);
+
 	/*
 	 * Don't allow the destination to overlap
 	 * with any of the source ranges
@@ -249,22 +249,27 @@ consolidate_set_destination (Consolidate *cs, Sheet *sheet, Range const *r)
 			return FALSE;
 		}
 	}
-	
+
 	if (cs->dst)
 		global_range_free (cs->dst);
+
 	cs->dst = new;
-	
+
 	return TRUE;
 }
 
 gboolean
-consolidate_add_source (Consolidate *cs, Sheet *sheet, Range const *r)
+consolidate_add_source (Consolidate *cs, Value *range)
 {
 	GlobalRange *tmp, *new;
 
 	g_return_val_if_fail (cs != NULL, FALSE);
-	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
-	g_return_val_if_fail (r != NULL, FALSE);
+	g_return_val_if_fail (range != NULL, FALSE);
+
+	new = g_new (GlobalRange, 1);
+
+	new->sheet = range->v_range.cell.a.sheet;
+	setup_range_from_value (&new->range, range, TRUE);
 
 	/*
 	 * Make sure the range that is added doesn't overlap
@@ -277,18 +282,18 @@ consolidate_add_source (Consolidate *cs, Sheet *sheet, Range const *r)
 	 * larger) we do a strict sanity check here.
 	 */
 	tmp = global_range_dup (cs->dst);
-	tmp->range.end.col = tmp->range.start.col + (r->end.col - r->start.col);
-	tmp->range.end.row = tmp->range.start.row + (r->end.row - r->start.row);
+	tmp->range.end.col = tmp->range.start.col +
+		(new->range.end.col - new->range.start.col);
+	tmp->range.end.row = tmp->range.start.row +
+		(new->range.end.row - new->range.start.row);
 
-	new = global_range_new (sheet, r);
-	
 	if (global_range_overlap (tmp, new)) {
 		global_range_free (new);
 		global_range_free (tmp);
 		return FALSE;
 	}
 	global_range_free (tmp);
-	
+
 	cs->src = g_slist_append (cs->src, new);
 
 	return TRUE;
@@ -314,7 +319,7 @@ cb_tree_free (Value const *key, TreeItem *ti, gpointer user_data)
 
 	if (ti->val) {
 		GSList *l;
-		
+
 		for (l = ti->val; l != NULL; l = l->next)
 			global_range_free ((GlobalRange *) l->data);
 
@@ -335,7 +340,7 @@ tree_free (GTree *tree)
 
 /**
  * retrieve_row_tree:
- * 
+ *
  * This routine traverses the whole source list
  * of regions and puts all rows in regions which
  * have a similar key (the key is the first column
@@ -361,7 +366,7 @@ retrieve_row_tree (Consolidate *cs)
 {
 	GTree *tree;
 	GSList *l;
-	
+
 	g_return_val_if_fail (cs != NULL, NULL);
 
 	tree = g_tree_new ((GCompareFunc) cb_value_compare);
@@ -378,21 +383,21 @@ retrieve_row_tree (Consolidate *cs)
 				GSList *granges;
 				TreeItem *ti;
 				Range s;
-			
+
 				ti = g_tree_lookup (tree, (Value *) v);
-				
+
 				if (ti)
 					granges = ti->val;
 				else
 					granges = NULL;
-					
+
 				s.start.row = s.end.row = row;
 				s.start.col = sgr->range.start.col + 1;
 				s.end.col   = sgr->range.end.col;
 
 				gr = global_range_new (sgr->sheet, &s);
 				granges = g_slist_append (granges, gr);
-				
+
 				/*
 				 * NOTE: There is no need to duplicate the value
 				 * as it will not change during the consolidation
@@ -403,7 +408,7 @@ retrieve_row_tree (Consolidate *cs)
 				        ti->key = v;
 				}
 				ti->val = granges;
-				
+
 				g_tree_insert (tree, (Value *) ti->key, ti);
 			}
 		}
@@ -420,18 +425,18 @@ retrieve_col_tree (Consolidate *cs)
 {
 	GTree *tree;
 	GSList *l;
-	
+
 	g_return_val_if_fail (cs != NULL, NULL);
 
 	tree = g_tree_new ((GCompareFunc) cb_value_compare);
-	
+
 	for (l = cs->src; l != NULL; l = l->next) {
 		GlobalRange const *sgr = l->data;
 		int col;
 
 		for (col = sgr->range.start.col; col <= sgr->range.end.col; col++) {
 			Value const *v = sheet_cell_get_value (sgr->sheet, col, sgr->range.start.row);
-			
+
 			if (v && v->type != VALUE_EMPTY) {
 				GlobalRange *gr;
 				GSList *granges;
@@ -439,12 +444,12 @@ retrieve_col_tree (Consolidate *cs)
 				Range s;
 
 				ti = g_tree_lookup (tree, (Value *) v);
-				
+
 				if (ti)
 					granges = ti->val;
 				else
 					granges = NULL;
-					
+
 				s.start.col = s.end.col = col;
 				s.start.row = sgr->range.start.row + 1;
 				s.end.row   = sgr->range.end.row;
@@ -459,15 +464,15 @@ retrieve_col_tree (Consolidate *cs)
 				 */
 				if (!ti) {
 					ti = g_new0 (TreeItem, 1);
-				        ti->key = v; 
+				        ti->key = v;
 				}
 				ti->val = granges;
-				
+
 				g_tree_insert (tree, (Value *) ti->key, ti);
 			}
 		}
 	}
-	
+
 	return tree;
 }
 
@@ -482,7 +487,7 @@ key_list_get (Consolidate *cs, gboolean is_cols)
 {
 	GSList *keys = NULL;
 	GSList *l;
-	
+
 	for (l = cs->src; l != NULL; l = l->next) {
 		GlobalRange *sgr = l->data;
 		int i = is_cols
@@ -514,7 +519,7 @@ key_list_get (Consolidate *cs, gboolean is_cols)
 				keys = g_slist_insert_sorted (keys, (Value *) v, (GCompareFunc) cb_value_compare);
 		}
 	}
-	
+
 	return keys;
 }
 
@@ -539,7 +544,7 @@ key_list_get (Consolidate *cs, gboolean is_cols)
  * destination FullRange. Currently no clipping is done on the destination
  * range so the dst->range->end.* are ignored. (clipping isn't terribly
  * useful either)
- **/ 
+ **/
 static void
 simple_consolidate (FunctionDefinition *fd, GlobalRange const *dst, GSList const *src,
 		    gboolean is_col_or_row, int *end_col, int *end_row)
@@ -549,7 +554,7 @@ simple_consolidate (FunctionDefinition *fd, GlobalRange const *dst, GSList const
 	Sheet *prev_sheet = NULL;
 	RangeRef *prev_r = NULL;
 	int x, y;
-	
+
 	g_return_if_fail (fd != NULL);
 	g_return_if_fail (dst != NULL);
 	g_return_if_fail (src != NULL);
@@ -563,16 +568,16 @@ simple_consolidate (FunctionDefinition *fd, GlobalRange const *dst, GSList const
 		box.end.row = SHEET_MAX_ROWS - 1;
 	if (dst->range.start.col + box.end.col >= SHEET_MAX_COLS)
 		box.end.col = SHEET_MAX_COLS - 1;
-		
+
 	for (y = box.start.row; y <= box.end.row; y++) {
 		for (x = box.start.col; x <= box.end.col; x++) {
-			GList *args = NULL;
-			
+			GnmExprList *args = NULL;
+
 			for (l = src; l != NULL; l = l->next) {
 				GlobalRange const *gr = l->data;
 				Value *val;
 				Range r;
-				
+
 				/*
 				 * We don't want to include this range
 				 * this time if the current traversal
@@ -610,20 +615,18 @@ simple_consolidate (FunctionDefinition *fd, GlobalRange const *dst, GSList const
 				val = value_new_cellrange_r (gr->sheet, &r);
 				prev_r = &val->v_range.cell;
 				prev_sheet = gr->sheet;
-				
-				args = g_list_append (args, expr_tree_new_constant (val));
+
+				args = gnm_expr_list_append (args, gnm_expr_new_constant (val));
 			}
-			
+
 			/* There is no need to free 'args', it will be absorbed
-			 * into the ExprTree
+			 * into the GnmExpr
 			 */
 			if (args) {
-				ExprTree *expr;
-
-				expr = expr_tree_new_funcall (fd, args);
+				GnmExpr const *expr = gnm_expr_new_funcall (fd, args);
 				set_cell_expr (dst->sheet, dst->range.start.col + x,
 					       dst->range.start.row + y, expr);
-				expr_tree_unref (expr);
+				gnm_expr_unref (expr);
 			}
 		}
 	}
@@ -637,6 +640,7 @@ simple_consolidate (FunctionDefinition *fd, GlobalRange const *dst, GSList const
 typedef struct {
 	Consolidate *cs;
 	GlobalRange *colrow;
+	WorkbookControl *wbc;
 } ConsolidateContext;
 
 /**
@@ -650,16 +654,16 @@ cb_row_tree (Value const *key, TreeItem *ti, ConsolidateContext *cc)
 {
 	Consolidate *cs;
 	int end_col = 0;
-	
+
 	cs = cc->cs;
 	if (cs->mode & CONSOLIDATE_COPY_LABELS)
 		set_cell_value (cs->dst->sheet, cc->colrow->range.start.col - 1,
 				cc->colrow->range.start.row, key);
 	simple_consolidate (cs->fd, cc->colrow, ti->val, FALSE, &end_col, NULL);
-	
+
 	if (end_col > cc->colrow->range.end.col)
 		cc->colrow->range.end.col = end_col;
-		
+
 	cc->colrow->range.start.row++;
 	cc->colrow->range.end.row++;
 
@@ -674,17 +678,17 @@ cb_row_tree (Value const *key, TreeItem *ti, ConsolidateContext *cc)
  * simple consolidation for each row.
  **/
 static void
-row_consolidate (Consolidate *cs)
+row_consolidate (Consolidate *cs, WorkbookView *wbv)
 {
 	ConsolidateContext cc;
 	GTree *tree;
-	
+
 	g_return_if_fail (cs != NULL);
 
 	tree = retrieve_row_tree (cs);
 	cc.cs = cs;
 	cc.colrow = global_range_dup (cs->dst);
-	
+
 	if (cs->mode & CONSOLIDATE_COPY_LABELS) {
 		cc.colrow->range.start.col++;
 		cc.colrow->range.end.col++;
@@ -692,14 +696,12 @@ row_consolidate (Consolidate *cs)
 
 	g_tree_traverse (tree, (GTraverseFunc) cb_row_tree,
 			 G_IN_ORDER, &cc);
-	
-	redraw_respan_and_select (cs->dst->sheet,
-				  cs->dst->range.start.col,
-				  cs->dst->range.start.row,
+
+	redraw_respan_and_select (cs->dst->sheet, &cs->dst->range.start,
 				  cc.colrow->range.end.col,
 				  cc.colrow->range.end.row - 1,
-				  cs->mode & CONSOLIDATE_PUT_VALUES);
-				 
+				  cs->mode & CONSOLIDATE_PUT_VALUES, wbv);
+
 	global_range_free (cc.colrow);
 	cc.colrow = NULL;
 
@@ -717,15 +719,15 @@ cb_col_tree (Value const *key, TreeItem *ti, ConsolidateContext *cc)
 {
 	Consolidate *cs;
 	int end_row = 0;
-	
+
 	cs = cc->cs;
-	
+
 	if (cs->mode & CONSOLIDATE_COPY_LABELS)
 		set_cell_value (cs->dst->sheet,
 				cc->colrow->range.start.col,
 				cc->colrow->range.start.row - 1,
 				key);
-			       
+
 	simple_consolidate (cs->fd, cc->colrow, ti->val, FALSE, NULL, &end_row);
 
 	if (end_row > cc->colrow->range.end.row)
@@ -745,11 +747,11 @@ cb_col_tree (Value const *key, TreeItem *ti, ConsolidateContext *cc)
  * simple consolidation for each column.
  **/
 static void
-col_consolidate (Consolidate *cs)
+col_consolidate (Consolidate *cs, WorkbookView *wbv)
 {
 	ConsolidateContext cc;
 	GTree *tree;
-	
+
 	g_return_if_fail (cs != NULL);
 	tree = retrieve_col_tree (cs);
 
@@ -763,13 +765,11 @@ col_consolidate (Consolidate *cs)
 
 	g_tree_traverse (tree, (GTraverseFunc) cb_col_tree,
 			 G_IN_ORDER, &cc);
-	
-	redraw_respan_and_select (cs->dst->sheet,
-				  cs->dst->range.start.col,
-				  cs->dst->range.start.row,
+
+	redraw_respan_and_select (cs->dst->sheet, &cs->dst->range.start,
 				  cc.colrow->range.end.col - 1,
 				  cc.colrow->range.end.row,
-				  cs->mode & CONSOLIDATE_PUT_VALUES);
+				  cs->mode & CONSOLIDATE_PUT_VALUES, wbv);
 
 	global_range_free (cc.colrow);
 	cc.colrow = NULL;
@@ -777,12 +777,12 @@ col_consolidate (Consolidate *cs)
 	tree_free (tree);
 }
 
-static GList *
+static GnmExprList *
 colrow_formula_args_build (Value const *row_name, Value const *col_name, GSList *granges)
 {
 	GSList const *l;
-	GList *args = NULL;
-	
+	GnmExprList *args = NULL;
+
 	for (l = granges; l != NULL; l = l->next) {
 		GlobalRange *gr = l->data;
 		int rx, ry;
@@ -802,7 +802,7 @@ colrow_formula_args_build (Value const *row_name, Value const *col_name, GSList
 			for (rx = gr->range.start.col + 1; rx <= gr->range.end.col; rx++) {
 				Value const *coltxt = sheet_cell_get_value (gr->sheet, rx, gr->range.start.row);
 				CellRef ref;
-				
+
 				if (coltxt == NULL || value_compare (coltxt, col_name, TRUE) != IS_EQUAL)
 					continue;
 
@@ -811,7 +811,7 @@ colrow_formula_args_build (Value const *row_name, Value const *col_name, GSList
 				ref.row = ry;
 				ref.col_relative = ref.row_relative = FALSE;
 
-				args = g_list_append (args, expr_tree_new_var (&ref));
+				args = gnm_expr_list_append (args, gnm_expr_new_cellref (&ref));
 			}
 
 		}
@@ -821,7 +821,7 @@ colrow_formula_args_build (Value const *row_name, Value const *col_name, GSList
 }
 
 static void
-colrow_consolidate (Consolidate *cs)
+colrow_consolidate (Consolidate *cs, WorkbookView *wbv)
 {
 	GSList *rows;
 	GSList *cols;
@@ -831,7 +831,7 @@ colrow_consolidate (Consolidate *cs)
 	int y = 0;
 
 	g_return_if_fail (cs != NULL);
-	
+
 	rows = key_list_get (cs, FALSE);
 	cols = key_list_get (cs, TRUE);
 
@@ -842,7 +842,7 @@ colrow_consolidate (Consolidate *cs)
 
 	for (l = rows; l != NULL && cs->dst->range.start.row + y < SHEET_MAX_ROWS; l = l->next, y++) {
 		Value const *row_name = l->data;
-		
+
 		if (cs->mode & CONSOLIDATE_COPY_LABELS) {
 			set_cell_value (cs->dst->sheet,
 					cs->dst->range.start.col,
@@ -854,7 +854,7 @@ colrow_consolidate (Consolidate *cs)
 
 		for (m = cols; m != NULL && cs->dst->range.start.col + x < SHEET_MAX_COLS; m = m->next, x++) {
 			Value const *col_name = m->data;
-			GList *args;
+			GnmExprList *args;
 
 			if (cs->mode & CONSOLIDATE_COPY_LABELS) {
 				set_cell_value (cs->dst->sheet,
@@ -866,21 +866,18 @@ colrow_consolidate (Consolidate *cs)
 			args = colrow_formula_args_build (row_name, col_name, cs->src);
 
 			if (args) {
-				ExprTree *expr = expr_tree_new_funcall (cs->fd, args);
-				
+				GnmExpr const *expr = gnm_expr_new_funcall (cs->fd, args);
 				set_cell_expr (cs->dst->sheet, cs->dst->range.start.col + x,
 					       cs->dst->range.start.row + y, expr);
-				expr_tree_unref (expr);
+				gnm_expr_unref (expr);
 			}
 		}
 	}
 
-	redraw_respan_and_select (cs->dst->sheet,
-				  cs->dst->range.start.col,
-				  cs->dst->range.start.row,
+	redraw_respan_and_select (cs->dst->sheet, &cs->dst->range.start,
 				  cs->dst->range.end.col + x - 1,
 				  cs->dst->range.end.row + y - 1,
-				  cs->mode & CONSOLIDATE_PUT_VALUES);
+				  cs->mode & CONSOLIDATE_PUT_VALUES, wbv);
 
 	/*
 	 * No need to free the values in these
@@ -900,11 +897,11 @@ Range
 consolidate_get_dest_bounding_box (Consolidate *cs)
 {
 	Range r;
-	
+
 	range_init (&r, 0, 0, 0, 0);
-	
+
 	g_return_val_if_fail (cs != NULL, r);
-	
+
 	if (cs->src)
 		get_bounding_box (cs->src, &r);
 
@@ -915,8 +912,10 @@ consolidate_get_dest_bounding_box (Consolidate *cs)
 }
 
 void
-consolidate_apply (Consolidate *cs)
+consolidate_apply (Consolidate *cs, WorkbookControl *wbc)
 {
+	WorkbookView *wbv = wb_control_view (wbc);
+
 	g_return_if_fail (cs != NULL);
 
 	/*
@@ -928,19 +927,17 @@ consolidate_apply (Consolidate *cs)
 		return;
 
 	if ((cs->mode & CONSOLIDATE_ROW_LABELS) && (cs->mode & CONSOLIDATE_COL_LABELS))
-		colrow_consolidate (cs);
+		colrow_consolidate (cs, wbv);
 	else if (cs->mode & CONSOLIDATE_ROW_LABELS)
-		row_consolidate (cs);
+		row_consolidate (cs, wbv);
 	else if (cs->mode & CONSOLIDATE_COL_LABELS)
-		col_consolidate (cs);
+		col_consolidate (cs, wbv);
 	else {
 		int end_row, end_col;
-		
+
 		simple_consolidate (cs->fd, cs->dst, cs->src, FALSE, &end_col, &end_row);
-		redraw_respan_and_select (cs->dst->sheet,
-					  cs->dst->range.start.col,
-					  cs->dst->range.start.row,
+		redraw_respan_and_select (cs->dst->sheet, &cs->dst->range.start,
 					  end_col, end_row,
-					  cs->mode & CONSOLIDATE_PUT_VALUES);
+					  cs->mode & CONSOLIDATE_PUT_VALUES, wbv);
 	}
 }
