@@ -1,9 +1,24 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * dialog-goal-seek.c:
  *
  * Authors:
  *   Jukka-Pekka Iivonen <iivonen@iki.fi>
  *   Morten Welinder (terra@diku.dk)
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include <gnumeric-config.h>
@@ -14,7 +29,7 @@
 #include <cell.h>
 #include <sheet.h>
 #include <expr.h>
-#include <eval.h>
+#include <dependent.h>
 #include <format.h>
 #include <value.h>
 #include <mstyle.h>
@@ -25,16 +40,12 @@
 #include <workbook-control.h>
 #include <workbook-edit.h>
 #include <workbook-view.h>
-#include <utils-dialog.h>
 #include <goal-seek.h>
 #include <mathfunc.h>
 #include <widgets/gnumeric-expr-entry.h>
 
 #include <libgnome/gnome-i18n.h>
 #include <math.h>
-#ifdef HAVE_IEEEFP_H
-#include <ieeefp.h>
-#endif
 
 #define MAX_CELL_NAME_LEN  20
 #define GOALSEEK_KEY            "goal-seek-dialog"
@@ -50,7 +61,6 @@ typedef struct {
 	GtkWidget *close_button;
 	GtkWidget *cancel_button;
 	GtkWidget *apply_button;
-	GtkWidget *help_button;
 	GtkWidget *target_value_label;
 	GtkWidget *current_value_label;
 	GtkWidget *solution_label;
@@ -66,37 +76,27 @@ typedef struct {
 	Cell *change_cell;
 	Cell *old_cell;
 	Value *old_value;
+	GtkWidget *warning_dialog;
 } GoalSeekState;
 
 
-static void
-focus_on_entry (GtkWidget *entry)
-{
-	gtk_widget_grab_focus (entry);
-	gtk_entry_set_position (GTK_ENTRY (entry), 0);
-	gtk_entry_select_region (GTK_ENTRY (entry), 0,
-				 GTK_ENTRY (entry)->text_length);
-}
-
-
 typedef struct {
 	Cell *xcell, *ycell;
 	gnum_float ytarget;
 } GoalEvalData;
 
-
 static GoalSeekStatus
 goal_seek_eval (gnum_float x, gnum_float *y, void *vevaldata)
 {
 	GoalEvalData *evaldata = vevaldata;
 
-	cell_set_value (evaldata->xcell, value_new_float (x), NULL);
+	cell_set_value (evaldata->xcell, value_new_float (x));
 	cell_queue_recalc (evaldata->xcell);
 	workbook_recalc (evaldata->xcell->base.sheet->workbook);
 
 	if (evaldata->ycell->value) {
 	        *y = value_get_as_float (evaldata->ycell->value) - evaldata->ytarget;
-		if (FINITE (*y))
+		if (finitegnum (*y))
 			return GOAL_SEEK_OK;
 	}
 
@@ -244,13 +244,6 @@ gnumeric_goal_seek (GoalSeekState *state)
 	return status;
 }
 
-
-static void
-dialog_help_cb (GtkWidget *button, char const *link)
-{
-	gnumeric_help_display (link);
-}
-
 /**
  * dialog_destroy:
  * @window:
@@ -265,16 +258,16 @@ dialog_destroy (GtkObject *w, GoalSeekState  *state)
 {
 	g_return_val_if_fail (w != NULL, FALSE);
 	g_return_val_if_fail (state != NULL, FALSE);
-	
+
 	wbcg_edit_detach_guru (state->wbcg);
 
 	if (state->old_value != NULL) {
 		value_release (state->old_value);
 		state->old_value = NULL;
 	}
-	
+
 	if (state->gui != NULL) {
-		gtk_object_unref (GTK_OBJECT (state->gui));
+		g_object_unref (G_OBJECT (state->gui));
 		state->gui = NULL;
 	}
 
@@ -298,7 +291,7 @@ static void
 cb_dialog_cancel_clicked (GtkWidget *button, GoalSeekState *state)
 {
 	if ((state->old_cell != NULL) && (state->old_value != NULL)) {
-		sheet_cell_set_value (state->old_cell, state->old_value, NULL);
+		sheet_cell_set_value (state->old_cell, state->old_value);
 		workbook_recalc (state->old_cell->base.sheet->workbook);
 		state->old_value = NULL;
 	}
@@ -330,131 +323,103 @@ cb_dialog_close_clicked (GtkWidget *button, GoalSeekState *state)
 static void
 cb_dialog_apply_clicked (GtkWidget *button, GoalSeekState *state)
 {
-	char const *text;
-	char *tmp;
 	char *status_str;
 	char *target_str;
 	char *actual_str;
 	char *solution_str;
 	GoalSeekStatus	status;
-	Value *value;
 	StyleFormat *format;
-	StyleFormat *target_value_format;
-	StyleFormat *min_value_format;
-	StyleFormat *max_value_format;
-  	gnum_float  max_range_val = 1e24;    
+  	gnum_float  max_range_val = 1e24;
 	Value *error_value, *target;
 	RangeRef const *r;
 
+	if (state->warning_dialog != NULL)
+		gtk_widget_destroy (state->warning_dialog);
+
 	/* set up source */
-	target = global_range_parse (state->sheet,
-		gtk_entry_get_text (GTK_ENTRY (state->set_cell_entry)));
+	target = gnm_expr_entry_parse_as_value (state->set_cell_entry, 
+						state->sheet);
 	if (target == NULL) {
-		gnumeric_notice (state->wbcg, GNOME_MESSAGE_BOX_ERROR,
-				 _("You should introduce a valid cell name in 'Set Cell:'!"));
-		focus_on_entry (GTK_WIDGET (state->set_cell_entry));
+		gnumeric_notice_nonmodal (GTK_WINDOW(state->dialog),
+					  &(state->warning_dialog),
+					  GTK_MESSAGE_ERROR,
+					  _("You should introduce a valid cell "
+					    "name in 'Set Cell:'!"));
+		gnm_expr_entry_grab_focus (state->set_cell_entry, TRUE);
 		return;
 	}
 	r = &target->v_range.cell;
 	state->set_cell = sheet_cell_get (r->a.sheet, r->a.col, r->a.row);
 	value_release (target);
 	if (state->set_cell == NULL || !cell_has_expr (state->set_cell)) {
-		gnumeric_notice (state->wbcg, GNOME_MESSAGE_BOX_ERROR,
-				 _("The cell named in 'Set Cell:' must contain a formula!"));
-		focus_on_entry (GTK_WIDGET (state->set_cell_entry));
+		gnumeric_notice_nonmodal (GTK_WINDOW(state->dialog),
+					  &(state->warning_dialog),
+					  GTK_MESSAGE_ERROR,
+					  _("The cell named in 'Set Cell:' "
+					    "must contain a formula!"));
+		gnm_expr_entry_grab_focus (state->set_cell_entry, TRUE);
 		return;
 	}
 
 	/* set up source */
-	target = global_range_parse (state->sheet,
-		gtk_entry_get_text (GTK_ENTRY (state->change_cell_entry)));
+	target = gnm_expr_entry_parse_as_value (state->change_cell_entry, 
+						state->sheet);
 	if (target == NULL) {
-		gnumeric_notice (state->wbcg, GNOME_MESSAGE_BOX_ERROR,
-				 _("You should introduce a valid cell "
-				   "name in 'By Changing Cell:'!"));
-		focus_on_entry (GTK_WIDGET (state->change_cell_entry));
+		gnumeric_notice_nonmodal (GTK_WINDOW(state->dialog),
+					  &(state->warning_dialog),
+					  GTK_MESSAGE_ERROR,
+					  _("You should introduce a valid cell "
+					    "name in 'By Changing Cell:'!"));
+		gnm_expr_entry_grab_focus (state->change_cell_entry, TRUE);
 		return;
 	}
 
 	r = &target->v_range.cell;
-	state->change_cell = sheet_cell_get (r->a.sheet, r->a.col, r->a.row);
+	state->change_cell = sheet_cell_fetch (r->a.sheet, r->a.col, r->a.row);
 	value_release (target);
 	if (cell_has_expr (state->change_cell)) {
-		gnumeric_notice (state->wbcg, GNOME_MESSAGE_BOX_ERROR,
-				 _("The cell named in 'By changing cell' "
-				   "must not contain a formula."));
-		focus_on_entry (GTK_WIDGET (state->change_cell_entry));
+		gnumeric_notice_nonmodal (GTK_WINDOW(state->dialog),
+					  &(state->warning_dialog),
+					  GTK_MESSAGE_ERROR,
+					  _("The cell named in 'By changing cell' "
+					    "must not contain a formula."));
+		gnm_expr_entry_grab_focus (state->change_cell_entry, TRUE);
 		return;
 	}
 
-	text = gtk_entry_get_text (GTK_ENTRY (state->to_value_entry));
+
 	format = mstyle_get_format (cell_get_mstyle (state->set_cell));
-	value = format_match_number (text, format, &target_value_format);
-	if (format != NULL) 
-		target_value_format = format;
-	if (value == NULL){
-		gnumeric_notice (state->wbcg, GNOME_MESSAGE_BOX_ERROR,
-				 _("The value given in 'To Value:' "
-				   "is not valid."));
-		focus_on_entry (GTK_WIDGET (state->to_value_entry));		
+	if (entry_to_float_with_format (GTK_ENTRY(state->to_value_entry), 
+					&state->target_value, TRUE, format)){
+		gnumeric_notice_nonmodal (GTK_WINDOW(state->dialog),
+					  &(state->warning_dialog),
+					  GTK_MESSAGE_ERROR,
+					  _("The value given in 'To Value:' "
+					    "is not valid."));
+		focus_on_entry (GTK_ENTRY(state->to_value_entry));
 		return;
 	}
-	state->target_value = value_get_as_float (value);
-	target_str = format_value (target_value_format, value, NULL, 0);
-	gtk_entry_set_text (GTK_ENTRY (state->to_value_entry), target_str);
-	g_free (target_str);
-	value_release (value);
-	
+
 	format = mstyle_get_format (cell_get_mstyle (state->change_cell));
-	tmp = g_strdup (gtk_entry_get_text (GTK_ENTRY (state->at_least_entry)));
-	g_strstrip (tmp);
-	if (strlen (tmp) > 0) {
-		value = format_match_number (tmp, format, &min_value_format);
-		if (format != NULL) 
-			min_value_format = format;
-	} else {
-		value = NULL;
-	}
-	g_free (tmp);
-	if (value != NULL) {
-		state->xmin = value_get_as_float (value);
-		tmp = format_value (min_value_format, value, NULL, 0);
-		gtk_entry_set_text (GTK_ENTRY (state->at_least_entry), tmp);
-		g_free (tmp);	
-		value_release (value);
-	} else {
+	if (entry_to_float_with_format (GTK_ENTRY(state->at_least_entry), 
+					 &state->xmin, TRUE, format)) {
 		state->xmin = - max_range_val;
 		gtk_entry_set_text (GTK_ENTRY (state->at_least_entry), "");
 	}
 
-	tmp = g_strdup (gtk_entry_get_text (GTK_ENTRY (state->at_most_entry)));
-	g_strstrip (tmp);
-	if (strlen (tmp) > 0) {
-		value = format_match_number (tmp, format, &max_value_format);
-		if (format != NULL) 
-			max_value_format = format;
-	} else {
-		value = NULL;
-	}
-	g_free (tmp);
-	if (value != NULL) {
-		state->xmax = value_get_as_float (value);
-		tmp = format_value (max_value_format, value, NULL, 0);
-		gtk_entry_set_text (GTK_ENTRY (state->at_most_entry), tmp);
-		g_free (tmp);	
-		value_release (value);
-	} else {
+	if (entry_to_float_with_format (GTK_ENTRY(state->at_most_entry), &state->xmax, 
+					TRUE, format)) {
   		state->xmax = max_range_val;
 		gtk_entry_set_text (GTK_ENTRY (state->at_most_entry), "");
 	}
 
 	if ((state->old_cell != NULL) && (state->old_value != NULL)) {
-		sheet_cell_set_value (state->old_cell, state->old_value, NULL);
+		sheet_cell_set_value (state->old_cell, state->old_value);
 		workbook_recalc (state->old_cell->base.sheet->workbook);
 		state->old_value = NULL;
 	}
 	state->old_cell = state->change_cell;
-	state->old_value = state->change_cell->value ? 
+	state->old_value = state->change_cell->value ?
 		value_duplicate (state->change_cell->value) : NULL;
 
 	status = gnumeric_goal_seek (state);
@@ -462,9 +427,9 @@ cb_dialog_apply_clicked (GtkWidget *button, GoalSeekState *state)
 	switch (status) {
 	case GOAL_SEEK_OK:
 		format = style_format_new_XL ("General", FALSE);
-		error_value = value_new_float (state->target_value - 
+		error_value = value_new_float (state->target_value -
 					      value_get_as_float (state->set_cell->value));
-  		target_str = format_value (format, error_value, NULL, 0);	 
+  		target_str = format_value (format, error_value, NULL, 0);
 		gtk_label_set_text (GTK_LABEL (state->target_value_label), target_str);
 		g_free (target_str);
 		value_release (error_value);
@@ -500,25 +465,6 @@ cb_dialog_apply_clicked (GtkWidget *button, GoalSeekState *state)
 	return;
 }
 
-/**
- * dialog_set_focus:
- * @window:
- * @focus_widget:
- * @state:
- *
- **/
-static void
-dialog_set_focus (GtkWidget *window, GtkWidget *focus_widget,
-			GoalSeekState *state)
-{
-	if (IS_GNUMERIC_EXPR_ENTRY (focus_widget)) {
-		wbcg_set_entry (state->wbcg,
-				    GNUMERIC_EXPR_ENTRY (focus_widget));
-		gnumeric_expr_entry_set_absolute (GNUMERIC_EXPR_ENTRY (focus_widget));
-	} else
-		wbcg_set_entry (state->wbcg, NULL);
-}
-
 /**
  * dialog_realized:
  * @widget
@@ -554,20 +500,23 @@ dialog_init (GoalSeekState *state)
                 return TRUE;
 
 	state->close_button     = glade_xml_get_widget (state->gui, "closebutton");
-	gtk_signal_connect (GTK_OBJECT (state->close_button), "clicked",
-			    GTK_SIGNAL_FUNC (cb_dialog_close_clicked),
-			    state);
+	g_signal_connect (G_OBJECT (state->close_button),
+		"clicked",
+		G_CALLBACK (cb_dialog_close_clicked), state);
 
 	state->cancel_button  = glade_xml_get_widget (state->gui, "cancelbutton");
-	gtk_signal_connect (GTK_OBJECT (state->cancel_button), "clicked",
-			    GTK_SIGNAL_FUNC (cb_dialog_cancel_clicked),
-			    state);
+	g_signal_connect (G_OBJECT (state->cancel_button),
+		"clicked",
+		G_CALLBACK (cb_dialog_cancel_clicked), state);
 	state->apply_button     = glade_xml_get_widget (state->gui, "applybutton");
-	gtk_signal_connect (GTK_OBJECT (state->apply_button), "clicked",
-			    GTK_SIGNAL_FUNC (cb_dialog_apply_clicked), state);
-	state->help_button     = glade_xml_get_widget (state->gui, "helpbutton");
-	gtk_signal_connect (GTK_OBJECT (state->help_button), "clicked",
-			    GTK_SIGNAL_FUNC (dialog_help_cb), "goal-seek.html");
+	g_signal_connect (G_OBJECT (state->apply_button),
+		"clicked",
+		G_CALLBACK (cb_dialog_apply_clicked), state);
+
+	gnumeric_init_help_button (
+		glade_xml_get_widget (state->gui, "helpbutton"),
+		"goal-seek.html");
+
 	state->to_value_entry = glade_xml_get_widget (state->gui, "to_value_entry");
 	state->at_least_entry = glade_xml_get_widget (state->gui, "at_least-entry");
 	state->at_most_entry = glade_xml_get_widget (state->gui, "at_most-entry");
@@ -582,46 +531,47 @@ dialog_init (GoalSeekState *state)
 	state->result_frame = glade_xml_get_widget (state->gui, "result-frame");
 
 	table = GTK_TABLE (glade_xml_get_widget (state->gui, "goal-table"));
-	state->set_cell_entry = GNUMERIC_EXPR_ENTRY (gnumeric_expr_entry_new (state->wbcg));
-	gnumeric_expr_entry_set_flags (state->set_cell_entry,
-                                      GNUM_EE_SINGLE_RANGE | GNUM_EE_SHEET_OPTIONAL, 
-                                      GNUM_EE_MASK);
-        gnumeric_expr_entry_set_scg (state->set_cell_entry, wb_control_gui_cur_sheet (state->wbcg));
+	state->set_cell_entry = gnumeric_expr_entry_new (state->wbcg, TRUE);
+	gnm_expr_entry_set_flags (state->set_cell_entry,
+		GNUM_EE_SINGLE_RANGE | GNUM_EE_SHEET_OPTIONAL | 
+				  GNUM_EE_ABS_ROW | GNUM_EE_ABS_COL,
+		GNUM_EE_MASK);
+        gnm_expr_entry_set_scg (state->set_cell_entry, wbcg_cur_scg (state->wbcg));
 	gtk_table_attach (table, GTK_WIDGET (state->set_cell_entry),
 			  1, 2, 0, 1,
 			  GTK_EXPAND | GTK_FILL, 0,
 			  0, 0);
  	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
-				  GTK_EDITABLE (state->set_cell_entry));
+				  GTK_WIDGET (state->set_cell_entry));
 	gtk_widget_show (GTK_WIDGET (state->set_cell_entry));
-	
-	state->change_cell_entry = GNUMERIC_EXPR_ENTRY (gnumeric_expr_entry_new (state->wbcg));
-	gnumeric_expr_entry_set_flags (state->change_cell_entry,
-				       GNUM_EE_SINGLE_RANGE | GNUM_EE_SHEET_OPTIONAL, 
-				       GNUM_EE_MASK);
-	gnumeric_expr_entry_set_scg (state->change_cell_entry, 
-				     wb_control_gui_cur_sheet (state->wbcg));
+
+	state->change_cell_entry = gnumeric_expr_entry_new (state->wbcg, TRUE);
+	gnm_expr_entry_set_flags (state->change_cell_entry,
+		GNUM_EE_SINGLE_RANGE | GNUM_EE_SHEET_OPTIONAL | 
+				  GNUM_EE_ABS_ROW | GNUM_EE_ABS_COL,
+		GNUM_EE_MASK);
+	gnm_expr_entry_set_scg (state->change_cell_entry, wbcg_cur_scg (state->wbcg));
 	gtk_table_attach (table, GTK_WIDGET (state->change_cell_entry),
 			  1, 2, 2, 3,
 			  GTK_EXPAND | GTK_FILL, 0,
 			  0, 0);
 	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
-				  GTK_EDITABLE (state->change_cell_entry));
-	gtk_widget_show (GTK_WIDGET (state->change_cell_entry));				
+				  GTK_WIDGET (state->change_cell_entry));
+	gtk_widget_show (GTK_WIDGET (state->change_cell_entry));
 
 
 	wbcg_edit_attach_guru (state->wbcg, state->dialog);
-	gtk_signal_connect (GTK_OBJECT (state->dialog), "set-focus",
-			    GTK_SIGNAL_FUNC (dialog_set_focus), state);
-	gtk_signal_connect (GTK_OBJECT (state->dialog), "realize",
-			    GTK_SIGNAL_FUNC (dialog_realized), state);
-	gtk_signal_connect (GTK_OBJECT (state->dialog), "destroy",
-			    GTK_SIGNAL_FUNC (dialog_destroy), state);
+	g_signal_connect (G_OBJECT (state->dialog),
+		"realize",
+		G_CALLBACK (dialog_realized), state);
+	g_signal_connect (G_OBJECT (state->dialog),
+		"destroy",
+		G_CALLBACK (dialog_destroy), state);
 
 	state->old_value = NULL;
 	state->old_cell = NULL;
 
-	gtk_widget_grab_focus (GTK_WIDGET (state->set_cell_entry));
+	gnm_expr_entry_grab_focus (state->set_cell_entry, FALSE);
 
 	return FALSE;
 }
@@ -652,9 +602,10 @@ dialog_goal_seek (WorkbookControlGUI *wbcg, Sheet *sheet)
 	state->wbcg  = wbcg;
 	state->wb   = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
 	state->sheet = sheet;
+	state->warning_dialog = NULL;
 
 	if (dialog_init (state)) {
-		gnumeric_notice (wbcg, GNOME_MESSAGE_BOX_ERROR,
+		gnumeric_notice (wbcg, GTK_MESSAGE_ERROR,
 				 _("Could not create the Goal-Seek dialog."));
 		g_free (state);
 		return;
