@@ -1,3 +1,4 @@
+/* vim: set sw=8: */
 /*
  * gui-file.c:
  *
@@ -18,9 +19,9 @@
 #include "workbook-control-gui-priv.h"
 #include "workbook-view.h"
 #include "workbook.h"
+#include "gnumeric-gconf.h"
 
 #include <libgnome/gnome-i18n.h>
-#include <libgnome/gnome-config.h>
 #include <libgnome/gnome-util.h>
 #include <errno.h>
 #include <sys/stat.h>
@@ -77,6 +78,42 @@ make_format_chooser (GList *list, GtkOptionMenu *omenu)
 	return (GTK_WIDGET (box));
 }
 
+gboolean
+gui_file_read (WorkbookControlGUI *wbcg, char const *file_name,
+	       GnumFileOpener const *optional_format)
+{
+	IOContext *io_context;
+	WorkbookView *wbv;
+
+	cmd_context_set_sensitive (COMMAND_CONTEXT (wbcg), FALSE);
+	io_context = gnumeric_io_context_new (COMMAND_CONTEXT (wbcg));
+	wbv = wb_view_new_from_file  (file_name, optional_format, io_context);
+
+	if (gnumeric_io_error_occurred (io_context))
+		gnumeric_io_error_display (io_context);
+
+	g_object_unref (G_OBJECT (io_context));
+	cmd_context_set_sensitive (COMMAND_CONTEXT (wbcg), TRUE);
+
+	if (wbv != NULL) {
+		Workbook *tmp_wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+		if (workbook_is_pristine (tmp_wb)) {
+			g_object_ref (G_OBJECT (wbcg));
+			workbook_unref (tmp_wb);
+			workbook_control_set_view (WORKBOOK_CONTROL (wbcg), wbv, NULL);
+			workbook_control_init_state (WORKBOOK_CONTROL (wbcg));
+		} else
+			(void) wb_control_wrapper_new (WORKBOOK_CONTROL (wbcg), wbv, NULL);
+
+		workbook_recalc (wb_view_workbook (wbv)); 
+		g_return_val_if_fail (!workbook_is_dirty (wb_view_workbook (wbv)), FALSE);
+
+		sheet_update (wb_view_cur_sheet (wbv));
+		return TRUE;
+	}
+	return FALSE;
+}
+
 /*
  * Lets the user choose an import filter for selected file, and
  * uses that to load the file.
@@ -91,12 +128,11 @@ gui_file_import (WorkbookControlGUI *wbcg)
 	GnumFileOpener *fo = NULL;
 	gchar const *file_name;
 
-	if (gnome_config_get_bool_with_default (
-	    "Gnumeric/File/ImportUsesAllOpeners=false", NULL)) {
+	if (gnm_gconf_get_import_uses_all_openers ())
 		importers = get_file_openers ();
-	} else {
+	else
 		importers = get_file_importers ();
-	}
+
 	importers = g_list_copy (importers);
 	importers = g_list_sort (importers, file_opener_description_cmp);
 
@@ -123,10 +159,9 @@ gui_file_import (WorkbookControlGUI *wbcg)
 
 	fo = g_list_nth_data (importers, gnumeric_option_menu_get_selected_index (omenu));
 	file_name = gtk_file_selection_get_filename (fsel);
-	if (fo != NULL) {
-		(void) wb_view_open_custom (wb_control_view (WORKBOOK_CONTROL (wbcg)),
-		                            WORKBOOK_CONTROL (wbcg), fo, file_name, TRUE);
-	}
+	if (fo != NULL)
+		gui_file_read (wbcg, file_name, fo);
+
 	gtk_object_destroy (GTK_OBJECT (fsel));
 	g_list_free (importers);
 }
@@ -165,9 +200,8 @@ can_try_save_to (WorkbookControlGUI *wbcg, const char *name)
 		msg = g_strdup_printf (
 		      _("Workbook %s already exists.\n"
 		      "Do you want to save over it?"), name);
-		result = gnumeric_dialog_question_yes_no (wbcg, msg,
-		         gnome_config_get_bool_with_default (
-		         "Gnumeric/File/FileOverwriteDefaultAnswer=false", NULL));
+		result = gnumeric_dialog_question_yes_no (
+			wbcg, msg, gnm_gconf_get_file_overwrite_default_answer());
 		g_free (msg);
 	}
 
@@ -182,7 +216,7 @@ check_multiple_sheet_support_if_needed (GnumFileSaver *fs,
 	gboolean ret_val = TRUE;
 
 	if (gnum_file_saver_get_save_scope (fs) == FILE_SAVE_SHEET &&
-	    gnome_config_get_bool_with_default ("Gnumeric/File/AskBeforeSavingOneSheet=true", NULL)) {
+	    gnm_gconf_get_file_ask_single_sheet_save ()) {
 		GList *sheets;
 		gchar *msg = _("Selected file format doesn't support "
 			       "saving multiple sheets in one file.\n"
@@ -305,29 +339,12 @@ void
 gui_file_open (WorkbookControlGUI *wbcg)
 {
 	GtkFileSelection *fsel;
-	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
-	Workbook *wb;
-	const gchar *wb_file_name;
 
 	fsel = GTK_FILE_SELECTION (gtk_file_selection_new (_("Load file")));
 	gtk_file_selection_hide_fileop_buttons (fsel);
 
-	/* Select current directory if we have one */
-	wb = wb_control_workbook (wbc);
-	wb_file_name = wb != NULL ? workbook_get_filename (wb) : NULL;
-	if (wb_file_name != NULL && strchr (wb_file_name, '/') != NULL) {
-		gchar *tmp, *dir_name;
-
-		tmp = g_dirname (wb_file_name);
-		dir_name = g_strconcat (tmp, "/", NULL);
-		gtk_file_selection_set_filename (fsel, dir_name);
-		g_free (dir_name);
-		g_free (tmp);
-	}
-
 	if (gnumeric_dialog_file_selection (wbcg, fsel))
-		(void) wb_view_open (wb_control_view (wbc), wbc,
-			gtk_file_selection_get_filename (fsel), TRUE);
+		gui_file_read (wbcg, gtk_file_selection_get_filename (fsel), NULL);
 
 	gtk_widget_destroy (GTK_WIDGET (fsel));
 }
@@ -348,7 +365,8 @@ gui_file_save (WorkbookControlGUI *wbcg, WorkbookView *wb_view)
 		return wb_view_save (wb_view, COMMAND_CONTEXT (wbcg));
 }
 
-#ifdef ENABLE_BONOBO
+#ifdef WITH_BONOBO
+#ifdef GNOME2_CONVERSION_COMPLETE
 static GnumFileSaver *
 ask_for_file_saver (WorkbookControlGUI *wbcg, WorkbookView *wb_view)
 {
@@ -357,19 +375,19 @@ ask_for_file_saver (WorkbookControlGUI *wbcg, WorkbookView *wb_view)
 	GtkOptionMenu *omenu;
 	GList *savers;
 	GnumFileSaver *fs;
-	const gchar *buttons[] = {GNOME_STOCK_BUTTON_OK,
-		                  GNOME_STOCK_BUTTON_CANCEL, NULL};
 
-	dialog = gnome_message_box_newv (_("Which file format would you like?"),
-					 GNOME_MESSAGE_BOX_QUESTION, buttons);
-	gnome_dialog_set_close (GNOME_DIALOG (dialog), FALSE);
+	dialog = gtk_message_dialog_new (wbcg_toplevel (wbcg),
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_OK_CANCEL,
+					 _("Which file format would you like?"));
 
 	/* Add the format chooser */
 	savers = g_list_copy (get_file_savers ());
 	savers = g_list_sort (savers, file_saver_description_cmp);
 	omenu = GTK_OPTION_MENU (gtk_option_menu_new ());
 	format_chooser = make_format_chooser (savers, omenu);
-	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    format_chooser, FALSE, FALSE, 0);
 
 	/* Set default file saver */
@@ -383,10 +401,10 @@ ask_for_file_saver (WorkbookControlGUI *wbcg, WorkbookView *wb_view)
 		fs = get_default_file_saver ();
 	}
 	gtk_option_menu_set_history (omenu, g_list_index (savers, fs));
-	gtk_widget_show_all (dialog);
+	gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
 
-	switch (gnome_dialog_run (GNOME_DIALOG (dialog))) {
-	case 0: /* Ok */
+	switch (gnumeric_dialog_run (wbcg, GTK_DIALOG (dialog))) {
+	case GTK_RESPONSE_OK: /* Ok */
 		fs = g_list_nth_data (savers,
 			gnumeric_option_menu_get_selected_index (omenu));
 		break;
@@ -394,7 +412,6 @@ ask_for_file_saver (WorkbookControlGUI *wbcg, WorkbookView *wb_view)
 		fs = NULL;
 		break;
 	}
-	gnome_dialog_close (GNOME_DIALOG (dialog));
 	g_list_free (savers);
 
 	return (fs);
@@ -437,4 +454,5 @@ gui_file_save_to_stream (BonoboStream *stream, WorkbookControlGUI *wbcg,
 	gtk_object_destroy (GTK_OBJECT (io_context));
 }
 #endif
+#endif
 
