@@ -5,32 +5,34 @@
  *  Mike Kestner (mkestner@ameritech.net)
  *
  */
-#include <config.h>
-#include <glib.h>
+#include <gnumeric-config.h>
+#include "gnumeric.h"
+#include "history.h"
 
 #include "application.h"
-#include "history.h"
+#include "gui-file.h"
 #include "workbook-control-gui-priv.h"
 #include "workbook-view.h"
 #include "workbook.h"
 
-#include <libgnome/gnome-defs.h>
+#include <gtk/gtkmenuitem.h>
 #include <libgnome/gnome-i18n.h>
-#ifdef ENABLE_BONOBO
+#include <libgnomeui/gnome-app-helper.h>
+#ifdef WITH_BONOBO
 #include <bonobo.h>
 #endif
+#include <string.h>
 
 /* Command callback called on activation of a file history menu item. */
-#ifndef ENABLE_BONOBO
+#ifndef WITH_BONOBO
 
 #define UGLY_GNOME_UI_KEY "HistoryFilename"
 
 static void
 file_history_cmd (GtkWidget *widget, WorkbookControlGUI *wbcg)
 {
-	gchar *filename = gtk_object_get_data (GTK_OBJECT (widget), UGLY_GNOME_UI_KEY);
-	(void) wb_view_open (wb_control_view (WORKBOOK_CONTROL (wbcg)),
-	                     WORKBOOK_CONTROL (wbcg), filename, TRUE);
+	char *filename = gtk_object_get_data (GTK_OBJECT (widget), UGLY_GNOME_UI_KEY);
+	gui_file_read (wbcg, filename, NULL);
 }
 
 #else
@@ -41,10 +43,9 @@ file_history_cmd (BonoboUIComponent *uic, WorkbookControlGUI *wbcg, const char *
 	char *filename = bonobo_ui_component_get_prop (wbcg->uic, fullpath,
 						       "tip", NULL);
 
-	g_free (fullpath);
-	(void) wb_view_open (wb_control_view (WORKBOOK_CONTROL (wbcg)),
-	                     WORKBOOK_CONTROL (wbcg), filename, TRUE);
+	gui_file_read (wbcg, filename, NULL);
 	g_free (filename);
+	g_free (fullpath);
 }
 #endif
 
@@ -79,18 +80,19 @@ history_item_label (const gchar *name, const gint accel_number)
 /*
  * Create a history menu item in a menu at a given position.
  */
-#ifndef ENABLE_BONOBO
+#ifndef WITH_BONOBO
 static void
 history_menu_item_create (WorkbookControlGUI *wbcg, gchar *name, gint accel_number,
 			  GtkWidget *menu, gint pos)
 {
+	char *label;
 	GnomeUIInfo info[] = {
 		{ GNOME_APP_UI_ITEM, NULL, NULL, file_history_cmd, NULL },
 		GNOMEUIINFO_END
 	};
 
 	info [0].hint = name;
-	info [0].label = history_item_label (name, accel_number);
+	info [0].label = label = history_item_label (name, accel_number);
 	info [0].user_data = wbcg;
 
 	gnome_app_fill_menu (GTK_MENU_SHELL (menu), info,
@@ -99,11 +101,11 @@ history_menu_item_create (WorkbookControlGUI *wbcg, gchar *name, gint accel_numb
 	gtk_object_set_data (GTK_OBJECT (info [0].widget), UGLY_GNOME_UI_KEY, name);
 	gnome_app_install_menu_hints (GNOME_APP (wbcg->toplevel), info);
 
-	g_free (info[0].label);
+	g_free (label);
 }
 #endif
 
-#ifndef ENABLE_BONOBO
+#ifndef WITH_BONOBO
 typedef struct {
 	GtkWidget *menu;
 	gint      pos;
@@ -116,52 +118,47 @@ typedef struct {
  * gnome_app_find_menu_pos returns the position *after* the menu item given.
  * bonobo_ui_handler_menu_get_pos returns the position *of* the menu item.
  */
-static MenuPos
-history_menu_locate_separator (WorkbookControlGUI *wbcg)
+static void
+history_menu_locate_separator (WorkbookControlGUI *wbcg, MenuPos *res)
 {
-	MenuPos ret;
 /*
  * xgettext:
  * This string must translate to exactly the same strings as the
- * 'Print Preview' item in the
+ * 'Print Preview...' item in the
  * 'File' menu
  */
-	char const * menu_name = _("File/Print Preview");
-	ret.menu = gnome_app_find_menu_pos (GNOME_APP (wbcg->toplevel)->menubar,
-					    menu_name, &ret.pos);
-	return ret;
+	char const *menu_name = _("_File/Print Pre_view...");
+	res->menu = gnome_app_find_menu_pos (GNOME_APP (wbcg->toplevel)->menubar,
+					    menu_name, &res->pos);
+	if (res->menu == NULL)
+		g_warning ("Probable mis-translation. '%s' : was not found. "
+			   "Does this match the '_File/Print Pre_view...' menu exactly ?",
+			   menu_name);
 }
 
 /* Insert the history separator. Return its position.  */
-static MenuPos
-history_menu_insert_separator (WorkbookControlGUI *wbcg)
+static void
+history_menu_insert_separator (WorkbookControlGUI *wbcg, MenuPos *res)
 {
-	MenuPos ret;
-
 	GtkWidget *item;
-	char const * menu_name = _("File/Print Preview");
 
-	ret = history_menu_locate_separator (wbcg);
-	if (ret.menu != NULL) {
+	history_menu_locate_separator (wbcg, res);
+	if (res->menu != NULL) {
 		item = gtk_menu_item_new ();
 		gtk_widget_show (item);
-		gtk_menu_shell_insert (GTK_MENU_SHELL (ret.menu), item, ret.pos);
+		gtk_menu_shell_insert (GTK_MENU_SHELL (res->menu), item, res->pos);
 		gtk_widget_set_sensitive(item, FALSE);
-	} else
-		g_warning ("Probable mis-translation. '%s' : was not found. "
-			   "Does this match the 'File/Print Preview' menu exactly ?",
-			   menu_name);
-	return ret;
+	}
 }
 
 /*
  * Add the history items list to the file menu for the workbook specified.
  */
 static void
-history_menu_insert_items (WorkbookControlGUI *wbcg, GList *name_list, MenuPos *mp)
+history_menu_insert_items (WorkbookControlGUI *wbcg, GSList *name_list, MenuPos *mp)
 {
 	gint  accel_number;
-	GList *l;
+	GSList *l;
 
 	g_return_if_fail (name_list != NULL);
 
@@ -173,11 +170,11 @@ history_menu_insert_items (WorkbookControlGUI *wbcg, GList *name_list, MenuPos *
 }
 #else
 static void
-history_menu_insert_items (WorkbookControlGUI *wbcg, GList *name_list)
+history_menu_insert_items (WorkbookControlGUI *wbcg, GSList *name_list)
 {
 	CORBA_Environment ev;
 	gint  accel_number;
-	GList *l;
+	GSList *l;
 
 	g_return_if_fail (name_list != NULL);
 
@@ -214,13 +211,13 @@ history_menu_insert_items (WorkbookControlGUI *wbcg, GList *name_list)
 }
 #endif
 
-#ifndef ENABLE_BONOBO
+#ifndef WITH_BONOBO
 /*
  * Remove the history list items from the file menu of the workbook
  * specified.
  */
 static void
-history_menu_remove_items (WorkbookControlGUI *wbcg, GList *name_list)
+history_menu_remove_items (WorkbookControlGUI *wbcg, GSList *name_list)
 {
 	gint  accel_number = 1;
 
@@ -230,7 +227,7 @@ history_menu_remove_items (WorkbookControlGUI *wbcg, GList *name_list)
 		label = history_item_label ((gchar *)name_list->data, accel_number);
 		path = g_strconcat (_("File/"), label, NULL);
 		gnome_app_remove_menus (GNOME_APP (wbcg->toplevel), path,
-					g_list_length (name_list));
+					g_slist_length (name_list));
 		g_free (label);
 		g_free (path);
 	}
@@ -242,10 +239,10 @@ history_menu_remove_items (WorkbookControlGUI *wbcg, GList *name_list)
  * specified.
  */
 static void
-history_menu_remove_items (WorkbookControlGUI *wbcg, GList *name_list)
+history_menu_remove_items (WorkbookControlGUI *wbcg, GSList *name_list)
 {
 	gint  accel_number = 1;
-	GList *l;
+	GSList *l;
 	CORBA_Environment ev;
 
 	CORBA_exception_init (&ev);
@@ -271,7 +268,7 @@ history_menu_remove_items (WorkbookControlGUI *wbcg, GList *name_list)
  * add a digit in front of the file name.
  */
 void
-history_menu_flush (GList *wl, GList *name_list)
+history_menu_flush (GList *wl, GSList *name_list)
 {
 	GList *l;
 
@@ -295,11 +292,11 @@ history_menu_flush (GList *wl, GList *name_list)
  * specified.
  */
 void
-history_menu_setup (WorkbookControlGUI *wbcg, GList *name_list)
+history_menu_setup (WorkbookControlGUI *wbcg, GSList *name_list)
 {
 	g_return_if_fail (name_list != NULL);
 
-#ifdef ENABLE_BONOBO
+#ifdef WITH_BONOBO
 	/* Insert the items */
 	history_menu_insert_items (wbcg, name_list);
 #else
@@ -307,7 +304,7 @@ history_menu_setup (WorkbookControlGUI *wbcg, GList *name_list)
 	MenuPos   mp;
 
 	/* Insert separator and get its position */
-	mp = history_menu_insert_separator (wbcg);
+	history_menu_insert_separator (wbcg, &mp);
 	(mp.pos)++;
 	/* Insert the items */
 	history_menu_insert_items (wbcg, name_list, &mp);
@@ -320,16 +317,16 @@ history_menu_setup (WorkbookControlGUI *wbcg, GList *name_list)
  * but the code is too messy to both with just now.
  */
 static void
-history_control_fill (WorkbookControl *control, GList *name_list, gboolean need_sep)
+history_control_fill (WorkbookControl *control, GSList *name_list, gboolean need_sep)
 {
 	if (IS_WORKBOOK_CONTROL_GUI (control)) {
 		WorkbookControlGUI *wbcg = WORKBOOK_CONTROL_GUI (control);
 		if (need_sep)
 			history_menu_setup (wbcg, name_list);
 		else {
-#ifndef ENABLE_BONOBO
+#ifndef WITH_BONOBO
 			MenuPos mp;
-			mp = history_menu_locate_separator (wbcg);
+			history_menu_locate_separator (wbcg, &mp);
 			(mp.pos)++;
 			/* Insert the items */
 			history_menu_insert_items (wbcg, name_list, &mp);
@@ -347,7 +344,7 @@ history_control_fill (WorkbookControl *control, GList *name_list, gboolean need_
  * Precondition - the old entries have been removed with history_menu_flush.
  */
 void
-history_menu_fill (GList *wl, GList *name_list,  gboolean need_sep)
+history_menu_fill (GList *wl, GSList *name_list,  gboolean need_sep)
 {
 	GList *l;
 
