@@ -28,7 +28,7 @@
 #include "parse-util.h"
 #include <libgnome/gnome-i18n.h>
 #include <gal/widgets/e-cursors.h>
-#include <gal/util/e-util.h>
+#include <gsf/gsf-impl-utils.h>
 #define GNUMERIC_ITEM "CURSOR"
 #include "item-debug.h"
 
@@ -739,7 +739,6 @@ item_cursor_selection_event (GnomeCanvasItem *item, GdkEvent *event)
 			return TRUE;
 
 		ic->drag_button = -1;
-
 		gnm_simple_canvas_ungrab (item, event->button.time);
 
 		if (sheet_is_region_empty (sheet, &ic->pos))
@@ -778,9 +777,10 @@ item_cursor_selection_event (GnomeCanvasItem *item, GdkEvent *event)
 
 			/* Make sure we don't overwrite the contents of the fill target */
 			for (x = ic->pos.end.col + 1; x <= final_col; x++) {
-				tmp = sheet_foreach_cell_in_range (
-					sheet, TRUE, x, ic->pos.start.row, x,
-					ic->pos.end.row, (ForeachCellCB) cb_autofill_bound,
+				tmp = sheet_foreach_cell_in_range (sheet,
+					CELL_ITER_IGNORE_BLANK,
+					x, ic->pos.start.row, x, ic->pos.end.row,
+					(CellIterFunc) cb_autofill_bound,
 					GINT_TO_POINTER (TRUE));
 
 				if (tmp) {
@@ -822,9 +822,11 @@ item_cursor_selection_event (GnomeCanvasItem *item, GdkEvent *event)
 			 * NOTE : We assume the traversal order is 'do all cols in a row and
 			 *        move on to the next row' here!
 			 */
-			tmp = sheet_foreach_cell_in_range (
-				sheet, TRUE, ic->pos.start.col, ic->pos.end.row + 1,
-				ic->pos.end.col, final_row, (ForeachCellCB) cb_autofill_bound,
+			tmp = sheet_foreach_cell_in_range (sheet,
+				CELL_ITER_IGNORE_BLANK,
+				ic->pos.start.col, ic->pos.end.row + 1,
+				ic->pos.end.col, final_row,
+				(CellIterFunc) cb_autofill_bound,
 				GINT_TO_POINTER (FALSE));
 
 			if (tmp) {
@@ -860,16 +862,18 @@ item_cursor_selection_event (GnomeCanvasItem *item, GdkEvent *event)
 		if (ic->drag_button >= 0)
 			return TRUE;
 
-		ic->drag_button = event->button.button;
-
-		/* prepare to create fill or drag cursors, but dont until we
-		 * move.  If we did create them here there would be problems
-		 * with race conditions when the new cursors pop into existence
-		 * during a double-click
-		 */
-		gnm_simple_canvas_grab (item,
-			GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK,
-			NULL, event->button.time);
+		if (event->button.button != 3) {
+			/* prepare to create fill or drag cursors, but dont until we
+			 * move.  If we did create them here there would be problems
+			 * with race conditions when the new cursors pop into existence
+			 * during a double-click
+			 */
+			ic->drag_button = event->button.button;
+			gnm_simple_canvas_grab (item,
+				GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK,
+				NULL, event->button.time);
+		} else
+			scg_context_menu (ic->scg, &event->button, FALSE, FALSE);
 		return TRUE;
 
 	case GDK_BUTTON_RELEASE:
@@ -878,8 +882,8 @@ item_cursor_selection_event (GnomeCanvasItem *item, GdkEvent *event)
 
 		/* Double clicks may have already released the drag prep */
 		if (ic->drag_button >= 0) {
-			ic->drag_button = -1;
 			gnm_simple_canvas_ungrab (item, event->button.time);
+			ic->drag_button = -1;
 		}
 		return TRUE;
 
@@ -897,7 +901,7 @@ item_cursor_target_region_ok (ItemCursor *ic)
 	g_return_val_if_fail (gci != NULL, FALSE);
 	g_return_val_if_fail (gci->canvas != NULL, FALSE);
 
-	if (sheet_is_region_empty_or_selected (sc->sheet, &ic->pos))
+	if (sv_is_region_empty_or_selected (sc->view, &ic->pos))
 		return TRUE;
 
 	return gnumeric_dialog_question_yes_no 
@@ -921,10 +925,11 @@ typedef enum {
 } ActionType;
 
 static void
-item_cursor_do_action (ItemCursor *ic, ActionType action, guint32 time)
+item_cursor_do_action (ItemCursor *ic, ActionType action)
 {
-	SheetControl *sc;
-	Sheet *sheet;
+	SheetControl	*sc;
+	SheetView 	*sv;
+	Sheet		*sheet;
 	WorkbookControl *wbc;
 	PasteTarget pt;
 
@@ -936,44 +941,41 @@ item_cursor_do_action (ItemCursor *ic, ActionType action, guint32 time)
 	}
 
 	sc = (SheetControl *) ic->scg;
-	sheet = sc->sheet;
-	wbc = sc->wbc;
+	sheet = sc_sheet (sc);
+	sv = sc_view (sc);
+	wbc = sc_wbc (sc);
 
 	switch (action) {
 	case ACTION_COPY_CELLS:
-		if (!sheet_selection_copy (wbc, sheet))
+		if (!sv_selection_copy (sv, wbc))
 			break;
 		cmd_paste (wbc,
 			   paste_target_init (&pt, sheet, &ic->pos,
-					      PASTE_ALL_TYPES),
-			   time);
+					      PASTE_ALL_TYPES));
 		break;
 
 	case ACTION_MOVE_CELLS:
-		if (!sheet_selection_cut (wbc, sheet))
+		if (!sv_selection_cut (sv, wbc))
 			break;
 		cmd_paste (wbc,
 			   paste_target_init (&pt, sheet, &ic->pos,
-					      PASTE_ALL_TYPES),
-			   time);
+					      PASTE_ALL_TYPES));
 		break;
 
 	case ACTION_COPY_FORMATS:
-		if (!sheet_selection_copy (wbc, sheet))
+		if (!sv_selection_copy (sv, wbc))
 			break;
 		cmd_paste (wbc,
 			   paste_target_init (&pt, sheet, &ic->pos,
-					      PASTE_FORMATS),
-			   time);
+					      PASTE_FORMATS));
 		break;
 
 	case ACTION_COPY_VALUES:
-		if (!sheet_selection_copy (wbc, sheet))
+		if (!sv_selection_copy (sv, wbc))
 			break;
 		cmd_paste (wbc,
 			   paste_target_init (&pt, sheet, &ic->pos,
-					      PASTE_AS_VALUES),
-			   time);
+					      PASTE_AS_VALUES));
 		break;
 
 	case ACTION_SHIFT_DOWN_AND_COPY:
@@ -996,7 +998,7 @@ context_menu_hander (GnumericPopupMenuElement const *element,
 {
 	g_return_val_if_fail (element != NULL, TRUE);
 
-	item_cursor_do_action (ic, element->index, GDK_CURRENT_TIME);
+	item_cursor_do_action (ic, element->index);
 	return TRUE;
 }
 
@@ -1046,8 +1048,8 @@ static void
 item_cursor_do_drop (ItemCursor *ic, GdkEventButton *event)
 {
 	/* Only do the operation if something moved */
-	Sheet const *sheet = ((SheetControl *) ic->scg)->sheet;
-	Range const *target = selection_first_range (sheet, NULL, NULL);
+	SheetView const *sv = ((SheetControl *) ic->scg)->view;
+	Range const *target = selection_first_range (sv, NULL, NULL);
 
 	wb_control_gui_set_status_text (ic->scg->wbcg, "");
 	if (range_equal (target, &ic->pos)) {
@@ -1059,8 +1061,8 @@ item_cursor_do_drop (ItemCursor *ic, GdkEventButton *event)
 		item_cursor_popup_menu (ic, event);
 	else
 		item_cursor_do_action (ic, (event->state & GDK_CONTROL_MASK)
-				       ? ACTION_COPY_CELLS : ACTION_MOVE_CELLS,
-				       event->time);
+				       ? ACTION_COPY_CELLS
+				       : ACTION_MOVE_CELLS);
 }
 
 static void
@@ -1421,6 +1423,6 @@ item_cursor_init (ItemCursor *ic)
 	ic->drag_button = -1;
 }
 
-E_MAKE_TYPE (item_cursor, "ItemCursor", ItemCursor,
-	     item_cursor_class_init, item_cursor_init,
-	     GNOME_TYPE_CANVAS_ITEM);
+GSF_CLASS (ItemCursor, item_cursor,
+	   item_cursor_class_init, item_cursor_init,
+	   GNOME_TYPE_CANVAS_ITEM);
