@@ -8,6 +8,7 @@
 #include "gnumeric.h"
 #include "plugin-loader-module.h"
 
+#include "gutils.h"
 #include "file.h"
 #include "plugin.h"
 #include "plugin-service.h"
@@ -39,9 +40,9 @@ struct _GnumericPluginLoaderModuleClass {
 
 static GnumericPluginLoaderClass *parent_class = NULL;
 
-static void gnumeric_plugin_loader_module_set_attributes (GnumericPluginLoader *loader, GSList *attr_names, GSList *attr_values, ErrorInfo **ret_error);
-static void gnumeric_plugin_loader_module_load (GnumericPluginLoader *loader, ErrorInfo **ret_error);
-static void gnumeric_plugin_loader_module_unload (GnumericPluginLoader *loader, ErrorInfo **ret_error);
+static void gnumeric_plugin_loader_module_set_attributes (GnumericPluginLoader *loader, GHashTable *attrs, ErrorInfo **ret_error);
+static void gnumeric_plugin_loader_module_load_base (GnumericPluginLoader *loader, ErrorInfo **ret_error);
+static void gnumeric_plugin_loader_module_unload_base (GnumericPluginLoader *loader, ErrorInfo **ret_error);
 
 static void gnumeric_plugin_loader_module_load_service_general (GnumericPluginLoader *loader, PluginService *service, ErrorInfo **ret_error);
 static void gnumeric_plugin_loader_module_load_service_file_opener (GnumericPluginLoader *loader, PluginService *service, ErrorInfo **ret_error);
@@ -49,34 +50,20 @@ static void gnumeric_plugin_loader_module_load_service_file_saver (GnumericPlugi
 static void gnumeric_plugin_loader_module_load_service_function_group (GnumericPluginLoader *loader, PluginService *service, ErrorInfo **ret_error);
 static void gnumeric_plugin_loader_module_unload_service_function_group (GnumericPluginLoader *loader, PluginService *service, ErrorInfo **ret_error);
 static void gnumeric_plugin_loader_module_load_service_plugin_loader (GnumericPluginLoader *loader, PluginService *service, ErrorInfo **ret_error);
+static void gnumeric_plugin_loader_module_load_service_ui (GnumericPluginLoader *loader, PluginService *service, ErrorInfo **ret_error);
+static void gnumeric_plugin_loader_module_unload_service_ui (GnumericPluginLoader *loader, PluginService *service, ErrorInfo **ret_error);
 
 
 static void
 gnumeric_plugin_loader_module_set_attributes (GnumericPluginLoader *loader,
-                                              GSList *attr_names, GSList *attr_values,
+                                              GHashTable *attrs,
                                               ErrorInfo **ret_error)
 {
-	GnumericPluginLoaderModule *loader_module;
+	GnumericPluginLoaderModule *loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
 	gchar *module_file_name = NULL;
-	GSList *ln, *lv;
-
-	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
-	g_return_if_fail (ret_error != NULL);
-
-	*ret_error = NULL;
-	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
-	for (ln = attr_names, lv = attr_values;
-	     ln != NULL && lv != NULL;
-	     ln = ln->next, lv = lv->next) {
-		gchar *name, *value;
-
-		name = (gchar *) ln->data;
-		value = (gchar *) lv->data;
-		if (strcmp ((gchar *) ln->data, "module_file") == 0) {
-			module_file_name = (gchar *) lv->data;
-		}
-	}
 
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	module_file_name = g_hash_table_lookup (attrs, "module_file");
 	if (module_file_name != NULL) {
 		loader_module->module_file_name = g_strdup (module_file_name);
 	} else {
@@ -86,22 +73,18 @@ gnumeric_plugin_loader_module_set_attributes (GnumericPluginLoader *loader,
 }
 
 static void
-gnumeric_plugin_loader_module_load (GnumericPluginLoader *loader, ErrorInfo **ret_error)
+gnumeric_plugin_loader_module_load_base (GnumericPluginLoader *loader, ErrorInfo **ret_error)
 {
-	GnumericPluginLoaderModule *loader_module;
+	GnumericPluginLoaderModule *loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
 
-	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
-	g_return_if_fail (ret_error != NULL);
-
-	*ret_error = NULL;
-	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
 	if (g_module_supported ()) {
 		gchar *full_module_file_name;
 		GModule *handle;
 		ModulePluginFileStruct *plugin_file_struct = NULL;
 		gpointer plugin_init_func = NULL, plugin_cleanup_func = NULL;
 
-		full_module_file_name = g_build_filename (plugin_info_peek_dir_name (loader->plugin),
+		full_module_file_name = g_build_filename (gnm_plugin_get_dir_name (loader->plugin),
 							  loader_module->module_file_name, NULL);
 		handle = g_module_open (full_module_file_name, 0);
 		if (handle != NULL) {
@@ -154,15 +137,11 @@ gnumeric_plugin_loader_module_load (GnumericPluginLoader *loader, ErrorInfo **re
 }
 
 static void
-gnumeric_plugin_loader_module_unload (GnumericPluginLoader *loader, ErrorInfo **ret_error)
+gnumeric_plugin_loader_module_unload_base (GnumericPluginLoader *loader, ErrorInfo **ret_error)
 {
-	GnumericPluginLoaderModule *loader_module;
+	GnumericPluginLoaderModule *loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
 
-	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
-	g_return_if_fail (ret_error != NULL);
-
-	*ret_error = NULL;
-	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
 	if (loader_module->plugin_cleanup_func != NULL) {
 		loader_module->plugin_cleanup_func ();
 	}
@@ -177,32 +156,6 @@ gnumeric_plugin_loader_module_unload (GnumericPluginLoader *loader, ErrorInfo **
 	loader_module->plugin_cleanup_func = NULL;
 }
 
-static gint
-gnumeric_plugin_loader_module_info_get_extra_info_list (GnumericPluginLoader *loader,
-                                                        GSList **ret_keys_list,
-                                                        GSList **ret_values_list)
-{
-	GnumericPluginLoaderModule *loader_module;
-	GSList *keys_list = NULL, *values_list = NULL;
-	gint n_items = 0;
-
-	g_return_val_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader), 0);
-	g_return_val_if_fail (ret_keys_list != NULL && ret_values_list != NULL, 0);
-
-	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
-	keys_list = g_slist_prepend (keys_list, g_strdup (_("Loader")));
-	values_list = g_slist_prepend (values_list, g_strdup ("g_module"));
-	n_items++;
-	keys_list = g_slist_prepend (keys_list, g_strdup (_("Module file name")));
-	values_list = g_slist_prepend (values_list, g_strdup (loader_module->module_file_name));
-	n_items++;
-
-	*ret_keys_list = g_slist_reverse (keys_list);
-	*ret_values_list = g_slist_reverse (values_list);
-
-	return n_items;
-}
-
 static void
 gnumeric_plugin_loader_module_init (GnumericPluginLoaderModule *loader_module)
 {
@@ -215,11 +168,8 @@ gnumeric_plugin_loader_module_init (GnumericPluginLoaderModule *loader_module)
 static void
 gnumeric_plugin_loader_module_destroy (GtkObject *obj)
 {
-	GnumericPluginLoaderModule *loader_module;
+	GnumericPluginLoaderModule *loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (obj);
 
-	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (obj));
-
-	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (obj);
 	g_free (loader_module->module_file_name);
 }
 
@@ -232,15 +182,16 @@ gnumeric_plugin_loader_module_class_init (GnumericPluginLoaderModuleClass *klass
 	parent_class = gtk_type_class (PARENT_TYPE);
 
 	gnumeric_plugin_loader_class->set_attributes = gnumeric_plugin_loader_module_set_attributes;
-	gnumeric_plugin_loader_class->load = gnumeric_plugin_loader_module_load;
-	gnumeric_plugin_loader_class->unload = gnumeric_plugin_loader_module_unload;
+	gnumeric_plugin_loader_class->load_base = gnumeric_plugin_loader_module_load_base;
+	gnumeric_plugin_loader_class->unload_base = gnumeric_plugin_loader_module_unload_base;
 	gnumeric_plugin_loader_class->load_service_general = gnumeric_plugin_loader_module_load_service_general;
 	gnumeric_plugin_loader_class->load_service_file_opener = gnumeric_plugin_loader_module_load_service_file_opener;
 	gnumeric_plugin_loader_class->load_service_file_saver = gnumeric_plugin_loader_module_load_service_file_saver;
 	gnumeric_plugin_loader_class->load_service_function_group = gnumeric_plugin_loader_module_load_service_function_group;
 	gnumeric_plugin_loader_class->unload_service_function_group = gnumeric_plugin_loader_module_unload_service_function_group;
 	gnumeric_plugin_loader_class->load_service_plugin_loader = gnumeric_plugin_loader_module_load_service_plugin_loader;
-	gnumeric_plugin_loader_class->get_extra_info_list = gnumeric_plugin_loader_module_info_get_extra_info_list;
+	gnumeric_plugin_loader_class->load_service_ui = gnumeric_plugin_loader_module_load_service_ui;
+	gnumeric_plugin_loader_class->unload_service_ui = gnumeric_plugin_loader_module_unload_service_ui;
 
 	gtk_object_class->destroy = gnumeric_plugin_loader_module_destroy;
 }
@@ -255,7 +206,6 @@ GSF_CLASS (GnumericPluginLoaderModule, gnumeric_plugin_loader_module,
 
 typedef struct {
 	void (*module_func_init) (ErrorInfo **ret_error);
-	gboolean (*module_func_can_deactivate) (void);
 	void (*module_func_cleanup) (ErrorInfo **ret_error);
 } ServiceLoaderDataGeneral;
 
@@ -265,37 +215,24 @@ gnumeric_plugin_loader_module_func_init (PluginService *service, ErrorInfo **ret
 	ErrorInfo *error;
 	ServiceLoaderDataGeneral *loader_data;
 
-	g_return_if_fail (service != NULL);
-	g_return_if_fail (ret_error != NULL);
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_GENERAL (service));
 
-	*ret_error = NULL;
-	loader_data = (ServiceLoaderDataGeneral *) plugin_service_get_loader_data (service);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
 	loader_data->module_func_init (&error);
 	*ret_error = error;
 }
 
-static gboolean
-gnumeric_plugin_loader_module_func_can_deactivate (PluginService *service)
-{
-	ServiceLoaderDataGeneral *loader_data;
-
-	g_return_val_if_fail (service != NULL, FALSE);
-
-	loader_data = (ServiceLoaderDataGeneral *) plugin_service_get_loader_data (service);
-	return loader_data->module_func_can_deactivate ();
-}
-
 static void
 gnumeric_plugin_loader_module_func_cleanup (PluginService *service, ErrorInfo **ret_error)
 {
 	ErrorInfo *error;
 	ServiceLoaderDataGeneral *loader_data;
 
-	g_return_if_fail (service != NULL);
-	g_return_if_fail (ret_error != NULL);
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_GENERAL (service));
 
-	*ret_error = NULL;
-	loader_data = (ServiceLoaderDataGeneral *) plugin_service_get_loader_data (service);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
 	loader_data->module_func_cleanup (&error);
 	*ret_error = error;
 }
@@ -305,32 +242,26 @@ gnumeric_plugin_loader_module_load_service_general (GnumericPluginLoader *loader
                                                     PluginService *service,
                                                     ErrorInfo **ret_error)
 {
-	GnumericPluginLoaderModule *loader_module;
-	gpointer module_func_init = NULL, module_func_cleanup = NULL, module_func_can_deactivate = NULL;
+	GnumericPluginLoaderModule *loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
+	gpointer module_func_init = NULL, module_func_cleanup = NULL;
 
-	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
-	g_return_if_fail (service != NULL);
-	g_return_if_fail (ret_error != NULL);
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_GENERAL (service));
 
-	*ret_error = NULL;
-	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
 	g_module_symbol (loader_module->handle, "plugin_init_general", &module_func_init);
 	g_module_symbol (loader_module->handle, "plugin_cleanup_general", &module_func_cleanup);
-	g_module_symbol (loader_module->handle, "plugin_can_deactivate_general", &module_func_can_deactivate);
-	if (module_func_init != NULL && module_func_cleanup != NULL && module_func_can_deactivate != NULL) {
-		PluginServiceGeneral *service_general;
+	if (module_func_init != NULL && module_func_cleanup != NULL) {
+		PluginServiceGeneralCallbacks *cbs;
 		ServiceLoaderDataGeneral *loader_data;
 
-		service_general = &service->t.general;
-		service_general->plugin_func_init = gnumeric_plugin_loader_module_func_init;
-		service_general->plugin_func_can_deactivate = gnumeric_plugin_loader_module_func_can_deactivate;
-		service_general->plugin_func_cleanup = gnumeric_plugin_loader_module_func_cleanup;
+		cbs = plugin_service_get_cbs (service);
+		cbs->plugin_func_init = gnumeric_plugin_loader_module_func_init;
+		cbs->plugin_func_cleanup = gnumeric_plugin_loader_module_func_cleanup;
 
 		loader_data = g_new (ServiceLoaderDataGeneral, 1);
 		loader_data->module_func_init = module_func_init;
 		loader_data->module_func_cleanup = module_func_cleanup;
-		loader_data->module_func_can_deactivate = module_func_can_deactivate;
-		plugin_service_set_loader_data (service, (gpointer) loader_data);
+		g_object_set_data (G_OBJECT (service), "loader_data", loader_data);
 	} else {
 		*ret_error = error_info_new_printf (
 		             _("Module file \"%s\" has invalid format."),
@@ -345,11 +276,6 @@ gnumeric_plugin_loader_module_load_service_general (GnumericPluginLoader *loader
 			                        error_info_new_str (
 			                        _("File doesn't contain \"plugin_cleanup_general\" function.")));
 		}
-		if (module_func_can_deactivate == NULL) {
-			error_info_add_details (*ret_error,
-			                        error_info_new_str (
-			                        _("File doesn't contain \"plugin_can_deactivate_general\" function.")));
-		}
 	}
 }
 
@@ -370,10 +296,10 @@ gnumeric_plugin_loader_module_func_file_probe (GnumFileOpener const *fo, PluginS
 {
 	ServiceLoaderDataFileOpener *loader_data;
 
-	g_return_val_if_fail (service != NULL, FALSE);
+	g_return_val_if_fail (GNM_IS_PLUGIN_SERVICE_FILE_OPENER (service), FALSE);
 	g_return_val_if_fail (input != NULL, FALSE);
 
-	loader_data = (ServiceLoaderDataFileOpener *) plugin_service_get_loader_data (service);
+	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
 	return loader_data->module_func_file_probe (fo, input, pl);
 }
 
@@ -384,10 +310,10 @@ gnumeric_plugin_loader_module_func_file_open (GnumFileOpener const *fo, PluginSe
 {
 	ServiceLoaderDataFileOpener *loader_data;
 
-	g_return_if_fail (service != NULL);
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_FILE_OPENER (service));
 	g_return_if_fail (input != NULL);
 
-	loader_data = (ServiceLoaderDataFileOpener *) plugin_service_get_loader_data (service);
+	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
 	loader_data->module_func_file_open (fo, io_context, wbv, input);
 }
 
@@ -396,58 +322,37 @@ gnumeric_plugin_loader_module_load_service_file_opener (GnumericPluginLoader *lo
                                                         PluginService *service,
                                                         ErrorInfo **ret_error)
 {
-	GnumericPluginLoaderModule *loader_module;
-	PluginServiceFileOpener *service_file_opener;
+	GnumericPluginLoaderModule *loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
 	gchar *func_name_file_probe, *func_name_file_open;
 	gpointer module_func_file_probe = NULL, module_func_file_open = NULL;
 
-	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
-	g_return_if_fail (service != NULL);
-	g_return_if_fail (ret_error != NULL);
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_FILE_OPENER (service));
 
-	*ret_error = NULL;
-	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
-	service_file_opener = &service->t.file_opener;
-	if (service_file_opener->has_probe) {
-		func_name_file_probe = g_strdup_printf ("%s_file_probe", service_file_opener->id);
-		g_module_symbol (loader_module->handle, func_name_file_probe, &module_func_file_probe);
-	} else {
-		func_name_file_probe = NULL;
-		module_func_file_probe = NULL;
-	}
-	func_name_file_open = g_strdup_printf ("%s_file_open", service_file_opener->id);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	func_name_file_probe = g_strconcat (
+		plugin_service_get_id (service), "_file_probe", NULL);
+	g_module_symbol (loader_module->handle, func_name_file_probe, &module_func_file_probe);
+	func_name_file_open = g_strconcat (
+		plugin_service_get_id (service), "_file_open", NULL);
 	g_module_symbol (loader_module->handle, func_name_file_open, &module_func_file_open);
-	if ((!service_file_opener->has_probe || module_func_file_probe != NULL) &&
-	    module_func_file_open != NULL) {
+	if (module_func_file_open != NULL) {
+		PluginServiceFileOpenerCallbacks *cbs;
 		ServiceLoaderDataFileOpener *loader_data;
 
-		if (service_file_opener->has_probe) {
-			service_file_opener->plugin_func_file_probe = gnumeric_plugin_loader_module_func_file_probe;
-		} else {
-			service_file_opener->plugin_func_file_probe = NULL;
-		}
-		service_file_opener->plugin_func_file_open = gnumeric_plugin_loader_module_func_file_open;
+		cbs = plugin_service_get_cbs (service);
+		cbs->plugin_func_file_probe = gnumeric_plugin_loader_module_func_file_probe;
+		cbs->plugin_func_file_open = gnumeric_plugin_loader_module_func_file_open;
 
 		loader_data = g_new (ServiceLoaderDataFileOpener, 1);
 		loader_data->module_func_file_probe = module_func_file_probe;
 		loader_data->module_func_file_open = module_func_file_open;
-		plugin_service_set_loader_data (service, (gpointer) loader_data);
+		g_object_set_data (G_OBJECT (service), "loader_data", loader_data);
 	} else {
 		*ret_error = error_info_new_printf (
 		             _("Module file \"%s\" has invalid format."),
 		             loader_module->module_file_name);
-		if (service_file_opener->has_probe && module_func_file_probe == NULL) {
-			error_info_add_details (*ret_error,
-			                        error_info_new_printf (
-			                        _("File doesn't contain \"%s\" function."),
-			                        func_name_file_probe));
-		}
-		if (module_func_file_open == NULL) {
-			error_info_add_details (*ret_error,
-			                        error_info_new_printf (
-			                        _("File doesn't contain \"%s\" function."),
-			                        func_name_file_open));
-		}
+		error_info_add_details (*ret_error, error_info_new_printf (
+			_("File doesn't contain \"%s\" function."), func_name_file_open));
 	}
 	g_free (func_name_file_probe);
 	g_free (func_name_file_open);
@@ -469,10 +374,10 @@ gnumeric_plugin_loader_module_func_file_save (GnumFileSaver const *fs, PluginSer
 {
 	ServiceLoaderDataFileSaver *loader_data;
 
-	g_return_if_fail (service != NULL);
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_FILE_SAVER (service));
 	g_return_if_fail (file_name != NULL);
 
-	loader_data = (ServiceLoaderDataFileSaver *) plugin_service_get_loader_data (service);
+	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
 	loader_data->module_func_file_save (fs, io_context, wbv, file_name);
 }
 
@@ -481,28 +386,26 @@ gnumeric_plugin_loader_module_load_service_file_saver (GnumericPluginLoader *loa
                                                        PluginService *service,
                                                        ErrorInfo **ret_error)
 {
-	GnumericPluginLoaderModule *loader_module;
-	PluginServiceFileSaver *service_file_saver;
+	GnumericPluginLoaderModule *loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
 	gchar *func_name_file_save;
 	gpointer module_func_file_save = NULL;
 
-	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
-	g_return_if_fail (service != NULL);
-	g_return_if_fail (ret_error != NULL);
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_FILE_SAVER (service));
 
-	*ret_error = NULL;
-	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
-	service_file_saver = &service->t.file_saver;
-	func_name_file_save = g_strdup_printf ("%s_file_save", service_file_saver->id);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	func_name_file_save = g_strconcat (
+		plugin_service_get_id (service), "_file_save", NULL);
 	g_module_symbol (loader_module->handle, func_name_file_save, &module_func_file_save);
 	if (module_func_file_save != NULL) {
+		PluginServiceFileSaverCallbacks *cbs;
 		ServiceLoaderDataFileSaver *loader_data;
 
-		service_file_saver->plugin_func_file_save = gnumeric_plugin_loader_module_func_file_save;
+		cbs = plugin_service_get_cbs (service);
+		cbs->plugin_func_file_save = gnumeric_plugin_loader_module_func_file_save;
 
 		loader_data = g_new (ServiceLoaderDataFileSaver, 1);
 		loader_data->module_func_file_save = module_func_file_save;
-		plugin_service_set_loader_data (service, (gpointer) loader_data);
+		g_object_set_data (G_OBJECT (service), "loader_data", loader_data);
 	} else {
 		*ret_error = error_info_new_printf (
 		             _("Module file \"%s\" has invalid format."),
@@ -540,10 +443,10 @@ gnumeric_plugin_loader_module_func_get_full_function_info (PluginService *servic
 	ServiceLoaderDataFunctionGroup *loader_data;
 	gpointer func_index_ptr;
 
-	g_return_val_if_fail (service != NULL, FALSE);
+	g_return_val_if_fail (GNM_IS_PLUGIN_SERVICE_FUNCTION_GROUP (service), FALSE);
 	g_return_val_if_fail (fn_name != NULL, FALSE);
 
-	loader_data = (ServiceLoaderDataFunctionGroup *) plugin_service_get_loader_data (service);
+	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
 	if (g_hash_table_lookup_extended (loader_data->function_indices, (gpointer) fn_name,
 	                                  NULL, &func_index_ptr)) {
 		ModulePluginFunctionInfo *fn_info;
@@ -567,25 +470,23 @@ gnumeric_plugin_loader_module_load_service_function_group (GnumericPluginLoader
                                                            PluginService *service,
                                                            ErrorInfo **ret_error)
 {
-	GnumericPluginLoaderModule *loader_module;
-	PluginServiceFunctionGroup *service_function_group;
+	GnumericPluginLoaderModule *loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
 	gchar *fn_info_array_name;
 	ModulePluginFunctionInfo *module_fn_info_array = NULL;
 
-	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
-	g_return_if_fail (service != NULL);
-	g_return_if_fail (ret_error != NULL);
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_FUNCTION_GROUP (service));
 
-	*ret_error = NULL;
-	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
-	service_function_group = &service->t.function_group;
-	fn_info_array_name = g_strdup_printf ("%s_functions", service_function_group->group_id);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	fn_info_array_name = g_strconcat (
+		plugin_service_get_id (service), "_functions", NULL);
 	g_module_symbol (loader_module->handle, fn_info_array_name, (gpointer) &module_fn_info_array);
 	if (module_fn_info_array != NULL) {
+		PluginServiceFunctionGroupCallbacks *cbs;
 		ServiceLoaderDataFunctionGroup *loader_data;
 		gint i;
 
-		service_function_group->plugin_func_get_full_function_info = &gnumeric_plugin_loader_module_func_get_full_function_info;
+		cbs = plugin_service_get_cbs (service);
+		cbs->plugin_func_get_full_function_info = &gnumeric_plugin_loader_module_func_get_full_function_info;
 
 		loader_data = g_new (ServiceLoaderDataFunctionGroup, 1);
 		loader_data->module_fn_info_array = module_fn_info_array;
@@ -595,7 +496,7 @@ gnumeric_plugin_loader_module_load_service_function_group (GnumericPluginLoader
 			                     (gpointer) module_fn_info_array[i].fn_name,
 			                     GINT_TO_POINTER (i));
 		}
-		plugin_service_set_loader_data (service, (gpointer) loader_data);
+		g_object_set_data (G_OBJECT (service), "loader_data", loader_data);
 	} else {
 		*ret_error = error_info_new_printf (
 		             _("Module file \"%s\" has invalid format."),
@@ -618,11 +519,10 @@ gnumeric_plugin_loader_module_unload_service_function_group (GnumericPluginLoade
 	ServiceLoaderDataFunctionGroup *loader_data;
 
 	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
-	g_return_if_fail (service != NULL);
-	g_return_if_fail (ret_error != NULL);
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_FUNCTION_GROUP (service));
 
-	*ret_error = NULL;
-	loader_data = (ServiceLoaderDataFunctionGroup *) plugin_service_get_loader_data (service);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
 	g_hash_table_destroy (loader_data->function_indices);
 	parent_class->unload_service_function_group (loader, service, ret_error);
 }
@@ -643,11 +543,10 @@ gnumeric_plugin_loader_module_func_get_loader_type (PluginService *service,
 	ErrorInfo *error;
 	GType loader_type;
 
-	g_return_val_if_fail (service != NULL, 0);
-	g_return_val_if_fail (ret_error != NULL, 0);
+	g_return_val_if_fail (GNM_IS_PLUGIN_SERVICE_PLUGIN_LOADER (service), 0);
 
-	*ret_error = NULL;
-	loader_data = (ServiceLoaderDataPluginLoader *) plugin_service_get_loader_data (service);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
 	loader_type = loader_data->module_func_get_loader_type (&error);
 	if (error == NULL) {
 		return loader_type;
@@ -662,37 +561,126 @@ gnumeric_plugin_loader_module_load_service_plugin_loader (GnumericPluginLoader *
                                                           PluginService *service,
                                                           ErrorInfo **ret_error)
 {
-	GnumericPluginLoaderModule *loader_module;
-	PluginServicePluginLoader *service_plugin_loader;
+	GnumericPluginLoaderModule *loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
 	gchar *func_name_get_loader_type;
 	gpointer module_func_get_loader_type = NULL;
 
-	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
-	g_return_if_fail (service != NULL);
-	g_return_if_fail (ret_error != NULL);
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_PLUGIN_LOADER (service));
 
-	*ret_error = NULL;
-	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
-	service_plugin_loader = &service->t.plugin_loader;
-	func_name_get_loader_type = g_strdup_printf ("%s_get_loader_type",
-	                                             service_plugin_loader->loader_id);
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	func_name_get_loader_type = g_strconcat (
+			plugin_service_get_id (service), "_get_loader_type", NULL);
 	g_module_symbol (loader_module->handle, func_name_get_loader_type,
 	                 &module_func_get_loader_type);
 	if (module_func_get_loader_type != NULL) {
+		PluginServicePluginLoaderCallbacks *cbs;
 		ServiceLoaderDataPluginLoader *loader_data;
 
-		service_plugin_loader->plugin_func_get_loader_type = gnumeric_plugin_loader_module_func_get_loader_type;
+		cbs = plugin_service_get_cbs (service);
+		cbs->plugin_func_get_loader_type = gnumeric_plugin_loader_module_func_get_loader_type;
 
 		loader_data = g_new (ServiceLoaderDataPluginLoader, 1);
 		loader_data->module_func_get_loader_type = module_func_get_loader_type;
-		plugin_service_set_loader_data (service, (gpointer) loader_data);
+		g_object_set_data (G_OBJECT (service), "loader_data", loader_data);
 	} else {
 		if (module_func_get_loader_type == NULL) {
-			error_info_add_details (*ret_error,
-			                        error_info_new_printf (
-			                        _("Module doesn't contain \"%s\" function."),
-			                        func_name_get_loader_type));
+			*ret_error = error_info_new_printf (
+			             _("Module doesn't contain \"%s\" function."),
+			             func_name_get_loader_type);
 		}
 	}
 	g_free (func_name_get_loader_type);
 }
+
+/*
+ * Service - ui
+ */
+
+typedef struct {
+	ModulePluginUIVerbInfo *module_ui_verbs_array;
+	GHashTable *ui_verbs_hash;
+} ServiceLoaderDataUI;
+
+static void
+gnumeric_plugin_loader_module_func_exec_verb (PluginService *service,
+                                              WorkbookControlGUI *wbcg,
+                                              BonoboUIComponent *uic,
+                                              const gchar *cname,
+                                              ErrorInfo **ret_error)
+{
+	ServiceLoaderDataUI *loader_data;
+	gpointer verb_index_ptr;
+	int verb_index;
+
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_UI (service));
+
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
+	if (!g_hash_table_lookup_extended (loader_data->ui_verbs_hash, cname,
+	                                   NULL, &verb_index_ptr)) {
+		*ret_error = error_info_new_printf (_("Unknown verb: %s"), cname);
+		return;
+	}
+	verb_index = GPOINTER_TO_INT (verb_index_ptr);
+	loader_data->module_ui_verbs_array[verb_index].verb_func (wbcg);
+}
+
+static void
+gnumeric_plugin_loader_module_load_service_ui (GnumericPluginLoader *loader,
+                                               PluginService *service,
+                                               ErrorInfo **ret_error)
+{
+	GnumericPluginLoaderModule *loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
+	char *ui_verbs_array_name;
+	ModulePluginUIVerbInfo *module_ui_verbs_array = NULL;
+	PluginServiceUICallbacks *cbs;
+	ServiceLoaderDataUI *loader_data;
+	gint i;
+
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_UI (service));
+
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	ui_verbs_array_name = g_strconcat (
+		plugin_service_get_id (service), "_ui_verbs", NULL);
+	g_module_symbol (loader_module->handle, ui_verbs_array_name, (gpointer) &module_ui_verbs_array);
+	if (module_ui_verbs_array == NULL) {
+		*ret_error = error_info_new_printf (
+			_("Module file \"%s\" has invalid format."),
+			loader_module->module_file_name);
+		error_info_add_details (*ret_error, error_info_new_printf (
+			_("File doesn't contain \"%s\" array."), ui_verbs_array_name));
+		g_free (ui_verbs_array_name);
+		return;
+	}
+	g_free (ui_verbs_array_name);
+
+	cbs = plugin_service_get_cbs (service);
+	cbs->plugin_func_exec_verb = gnumeric_plugin_loader_module_func_exec_verb;
+
+	loader_data = g_new (ServiceLoaderDataUI, 1);
+	loader_data->module_ui_verbs_array = module_ui_verbs_array;
+	loader_data->ui_verbs_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	for (i = 0; module_ui_verbs_array[i].verb_name != NULL; i++) {
+		g_hash_table_insert (
+			loader_data->ui_verbs_hash,
+			(gpointer) module_ui_verbs_array[i].verb_name,
+			GINT_TO_POINTER (i));
+	}
+	g_object_set_data (G_OBJECT (service), "loader_data", loader_data);
+}
+
+static void
+gnumeric_plugin_loader_module_unload_service_ui (GnumericPluginLoader *loader,
+                                                 PluginService *service,
+                                                 ErrorInfo **ret_error)
+{
+	ServiceLoaderDataUI *loader_data;
+
+	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
+	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_UI (service));
+
+	GNM_INIT_RET_ERROR_INFO (ret_error);
+	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
+	g_hash_table_destroy (loader_data->ui_verbs_hash);
+	parent_class->unload_service_ui (loader, service, ret_error);
+}
