@@ -15,8 +15,10 @@
 #include "sheet-object-impl.h"
 #include "selection.h"
 #include "workbook.h"
+#include "workbook-control.h"
 #include "dialogs.h"
-#include "main.h"
+#include "gnumeric-gconf.h"
+#include "libgnumeric.h"
 #include "sheet.h"
 #include "value.h"
 #include "cellspan.h"
@@ -32,12 +34,7 @@
 #include <libgnomeprint/gnome-print-config.h>
 #include <libgnomeprintui/gnome-print-master-preview.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
-
-#ifdef ENABLE_BONOBO
-#	include <bonobo/bonobo-print-client.h>
-#endif
-
-extern int print_debugging;
+#include <libgnomeprintui/gnome-printer-dialog.h>
 
 /*
  * Margins
@@ -276,10 +273,10 @@ static void
 print_hf_element (PrintJobInfo const *pj, char const *format,
 		  HFSide side, double y)
 {
-	PrintMargins *pm;
 	char *text;
 	double x;
 	double len;
+	double header = 0, footer = 0, left = 0, right = 0;
 
 	/* Be really really anal in case Bug
 	 * http://bugs.gnome.org/db/82/8200.html exists
@@ -299,18 +296,18 @@ print_hf_element (PrintJobInfo const *pj, char const *format,
 
 	len = get_width_string (pj->decoration_font, text);
 
-	pm = &pj->pi->margins;
+	print_info_get_margins   (pj->pi, &header, &footer, &left, &right);	
 	switch (side){
 	case LEFT_HEADER:
-		x = pm->left.points;
+		x = left;
 		break;
 
 	case RIGHT_HEADER:
-		x = pj->width - pm->right.points - len;
+		x = pj->width - right - len;
 		break;
 
 	case MIDDLE_HEADER:
-		x = (pj->x_points - len)/2 + pm->left.points;
+		x = (pj->x_points - len)/2 + left;
 		break;
 
 	default:
@@ -388,15 +385,18 @@ print_headers (PrintJobInfo const *pj)
 {
 	PrintMargins const *pm = &pj->pi->margins;
 	double top, bottom, y, ascender;
+	double header = 0, footer = 0, left = 0, right = 0;
+
+	print_info_get_margins   (pj->pi, &header, &footer, &left, &right);	
 
 	ascender = gnome_font_get_ascender (pj->decoration_font);
-	y = pj->height - pm->header.points - ascender;
-	top    =  1 + pj->height - MIN (pm->header.points, pm->top.points);
-	bottom = -1 + pj->height - MAX (pm->header.points, pm->top.points);
+	y = pj->height - header - ascender;
+	top    =  1 + pj->height - MIN (header, pm->top.points);
+	bottom = -1 + pj->height - MAX (header, pm->top.points);
 
 	print_hf_line (pj, pj->pi->header, y,
-		       -1 + pm->left.points, bottom,
-		       pj->width - pm->right.points, top);
+		       -1 + left, bottom,
+		       pj->width - right, top);
 }
 
 /*
@@ -411,33 +411,21 @@ print_footers (PrintJobInfo const *pj)
 {
 	PrintMargins *pm = &pj->pi->margins;
 	double top, bottom, y;
+	double header = 0, footer = 0, left = 0, right = 0;
 
-	y = pm->footer.points
-		+ gnome_font_get_descender (pj->decoration_font);
-	top    =  1 + MAX (pm->footer.points, pm->bottom.points);
-	bottom = -1 + MIN (pm->footer.points, pm->bottom.points);
+	print_info_get_margins   (pj->pi, &header, &footer, &left, &right);	
+
+	y = footer
+		- gnome_font_get_descender (pj->decoration_font);
+	top    =  1 + MAX (footer, pm->bottom.points);
+	bottom = -1 + MIN (footer, pm->bottom.points);
 
 	/* Clip path for the header
 	 * NOTE : postscript clip paths exclude the border, gdk includes it.
 	 */
 	print_hf_line (pj, pj->pi->footer, y,
-		       -1 + pm->left.points, bottom,
-		       pj->width - pm->right.points, top);
-}
-
-static void
-setup_rotation (PrintJobInfo const *pj)
-{
-	double affine [6];
-
-	if (pj->pi->orientation == PRINT_ORIENT_VERTICAL)
-		return;
-
-	art_affine_rotate (affine, 90.0);
-	gnome_print_concat (pj->print_context, affine);
-
-	art_affine_translate (affine, 0, -pj->height);
-	gnome_print_concat (pj->print_context, affine);
+		       -1 + left, bottom,
+		       pj->width - right, top);
 }
 
 static void
@@ -489,6 +477,7 @@ print_page (PrintJobInfo const *pj, Sheet const *sheet, Range *range,
 	char *pagenotxt;
 	gboolean printed;
 	GList *l;
+	double header = 0, footer = 0, left = 0, right = 0;
 
 	/* FIXME: Can col / row space calculation be factored out? */
 
@@ -574,9 +563,10 @@ print_page (PrintJobInfo const *pj, Sheet const *sheet, Range *range,
 		x = (pj->x_points - w)/2;
 	}
 
+	print_info_get_margins   (pj->pi, &header, &footer, &left, &right);
 	/* Margins */
-	x += margins->left.points;
-	y += MAX (margins->top.points, margins->header.points);
+	x += left;
+	y += MAX (margins->top.points, header);
 	if (pj->pi->print_grid_lines) {
 		/* the initial grid lines */
 		x += 1.;
@@ -595,7 +585,6 @@ print_page (PrintJobInfo const *pj, Sheet const *sheet, Range *range,
 	gnome_print_beginpage (pj->print_context, pagenotxt);
 	g_free (pagenotxt);
 
-	setup_rotation (pj);
 	print_headers (pj);
 	print_footers (pj);
 
@@ -919,7 +908,7 @@ compute_sheet_pages (PageCountInfo *pc, Sheet const *sheet)
 		r = pc->r;
 
 	/* Find out how many pages actually contain data */
-	pc->pages = print_sheet_range (pj, sheet, &r, FALSE);
+	pc->pages += print_sheet_range (pj, sheet, &r, FALSE);
 }
 
 /*
@@ -983,7 +972,8 @@ sheet_print_selection (PrintJobInfo *pj, Sheet const *sheet,
 	Range const *sel;
 	Range extent;
 
-	if (!(sel = selection_first_range (sheet, wbc, _("Print Region"))))
+	if (!(sel = selection_first_range (sheet_get_view (sheet, wb_control_view (wbc)),
+					   wbc, _("Print Region"))))
 		return;
 
 	extent = *sel;
@@ -1007,13 +997,51 @@ workbook_print_all (PrintJobInfo *pj, Workbook *wb)
 	g_list_free (sheets);
 }
 
+static void
+print_job_info_set_one_time_defaults (PrintJobInfo *pj) 
+{
+	char *str;
+	str = gnm_gconf_get_printer_backend ();
+	gnome_print_config_set (pj->pi->print_config, "Settings.Transport.Backend", str);
+	g_free (str);
+	str = gnm_gconf_get_printer_filename ();
+	gnome_print_config_set (pj->pi->print_config, "Settings.Transport.Backend.FileName", str);
+	g_free (str);
+	str = gnm_gconf_get_printer_command ();
+	gnome_print_config_set (pj->pi->print_config, "Settings.Transport.Backend.Command", str);
+	g_free (str);
+	str = gnm_gconf_get_printer_lpr_P ();
+	gnome_print_config_set (pj->pi->print_config, "Settings.Transport.Backend.Printer", str);
+	g_free (str);
+	str = gnm_gconf_get_printer ();
+	gnome_print_config_set (pj->pi->print_config, "Printer", str);
+	g_free (str);
+}
+
+static void
+print_job_info_save_one_time_defaults (PrintJobInfo *pj) 
+{
+	gnm_gconf_set_printer_backend 
+		(gnome_print_config_get (pj->pi->print_config, "Settings.Transport.Backend"));
+	gnm_gconf_set_printer_filename
+		(gnome_print_config_get (pj->pi->print_config, "Settings.Transport.Backend.FileName"));
+	gnm_gconf_set_printer_command
+		(gnome_print_config_get (pj->pi->print_config, "Settings.Transport.Backend.Command"));
+	gnm_gconf_set_printer_lpr_P
+		(gnome_print_config_get (pj->pi->print_config, "Settings.Transport.Backend.Printer"));
+	gnm_gconf_set_printer
+		(gnome_print_config_get (pj->pi->print_config, "Printer"));
+}
+
+
 static PrintJobInfo *
 print_job_info_get (Sheet *sheet, PrintRange range, gboolean const preview)
 {
 	GnomeFontFace *face;
 	PrintJobInfo *pj;
 	PrintMargins *pm = &sheet->print_info->margins;
-	int width, height;
+	double width = 1.0, height = 1.0;
+	double header = 0, footer = 0, left = 0, right = 0;
 
 	pj = g_new0 (PrintJobInfo, 1);
 
@@ -1021,7 +1049,9 @@ print_job_info_get (Sheet *sheet, PrintRange range, gboolean const preview)
 	 * Handy pointers
 	 */
 	pj->sheet = sheet;
-	pj->pi    = sheet->print_info;
+	pj->pi    = print_info_dup (sheet->print_info);
+
+	print_job_info_set_one_time_defaults (pj);
 
 	/*
 	 * Values that should be entered in a dialog box
@@ -1034,21 +1064,17 @@ print_job_info_get (Sheet *sheet, PrintRange range, gboolean const preview)
 	pj->current_output_sheet = 0;
 
 	/* Precompute information */
-	width  = pj->pi->paper->width;
-	height = pj->pi->paper->height;
+	gnome_print_master_get_page_size_from_config (pj->pi->print_config, 
+						      &width, &height);
+	pj->width = width;
+	pj->height = height;
 
-	if (pj->pi->orientation == PRINT_ORIENT_HORIZONTAL) {
-		pj->width = height;
-		pj->height = width;
-	} else {
-		pj->width = width;
-		pj->height = height;
-	}
 
-	pj->x_points = pj->width - (pm->left.points + pm->right.points);
+	print_info_get_margins   (pj->pi, &header, &footer, &left, &right);	
+	pj->x_points = pj->width - (left + right);
 	pj->y_points = pj->height -
-		(MAX (pm->top.points, pm->header.points) +
-		 MAX (pm->bottom.points, pm->footer.points));
+		(MAX (pm->top.points, header) +
+		 MAX (pm->bottom.points, footer));
 
 	/*
 	 * Setup render info
@@ -1068,8 +1094,8 @@ static void
 print_job_info_destroy (PrintJobInfo *pj)
 {
 	hf_render_info_destroy (pj->render_info);
-	gtk_object_unref (GTK_OBJECT (pj->decoration_font));
-
+	g_object_unref (G_OBJECT (pj->decoration_font));
+	print_info_free (pj->pi);
 	g_free (pj);
 }
 
@@ -1077,58 +1103,64 @@ void
 sheet_print (WorkbookControlGUI *wbcg, Sheet *sheet,
 	     gboolean preview, PrintRange default_range)
 {
-#if 0
-	GnomePrinter *printer = NULL;
-	PrintJobInfo *pj;
-	GnomePrintDialog *gpd;
-	GnomePrintMaster *gpm;
-	GnomePrintMasterPreview *pmp;
-	GnomePrintConfig *print_config;
+	PrintJobInfo *pj = NULL;
+	GtkWidget *gnome_print_dialog;
+	GnomePrintMaster *gpm = NULL;
 	int copies = 1;
 	int collate = FALSE;
  	int first = 1;
 	int end;
 	int range;
+	GtkWindow *toplevel;
+	GnomePrintConfig *print_config;
 
   	g_return_if_fail (IS_SHEET (sheet));
 
 	end  = workbook_sheet_count (sheet->workbook);
+	
+	pj = print_job_info_get (sheet, default_range, preview);
+
+	print_config = pj->pi->print_config;
+	pj->sorted_print = FALSE;
 
   	if (!preview) {
-		gpd = (GnomePrintDialog *)gnome_print_dialog_new (
-			_("Print Sheets"),
-			GNOME_PRINT_DIALOG_RANGE|GNOME_PRINT_DIALOG_COPIES);
+		gnome_print_dialog = g_object_new (GNOME_TYPE_PRINT_DIALOG,
+						   "print_config", print_config,
+						   NULL); 
 
-		g_return_if_fail (gpd != NULL);
+		g_return_if_fail (gnome_print_dialog != NULL);
+
+		gnome_print_dialog_construct (GNOME_PRINT_DIALOG (gnome_print_dialog), 
+					      _("Print Sheets"),
+					      GNOME_PRINT_DIALOG_RANGE |
+					      GNOME_PRINT_DIALOG_COPIES);
 
 		gnome_print_dialog_construct_range_page (
-			gpd,
-			GNOME_PRINT_RANGE_CURRENT|GNOME_PRINT_RANGE_ALL|
-			GNOME_PRINT_RANGE_SELECTION|GNOME_PRINT_RANGE_RANGE,
-			1, workbook_sheet_count(sheet->workbook),
-			_("Act_ive sheet"), _("S_heets"));
-#warning FIXME
-#if 0
-		gnome_dialog_set_default (GNOME_DIALOG (gpd),
-					  GNOME_PRINT_PRINT);
-		switch (gnumeric_dialog_run (wbcg, GNOME_DIALOG(gpd))) {
-		case GNOME_PRINT_PRINT:
+			GNOME_PRINT_DIALOG (gnome_print_dialog),
+			GNOME_PRINT_RANGE_CURRENT | GNOME_PRINT_RANGE_ALL |
+			GNOME_PRINT_RANGE_SELECTION | GNOME_PRINT_RANGE_RANGE,
+			first, end, _("Act_ive sheet"), _("S_heets"));
+
+		toplevel = wbcg_toplevel (wbcg);
+		if (GTK_WINDOW (gnome_print_dialog)->transient_parent != toplevel)
+			gtk_window_set_transient_for (GTK_WINDOW (gnome_print_dialog), toplevel);
+		
+		switch (gtk_dialog_run (GTK_DIALOG(gnome_print_dialog))) {
+		case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
 			break;
-		case GNOME_PRINT_PREVIEW:
+		case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
 			preview = TRUE;
 			break;
 		case -1:
   			return;
 		default:
-			gnome_dialog_close (GNOME_DIALOG (gpd));
+			gtk_widget_destroy (gnome_print_dialog);
 			return;
 		}
-#endif
-		gnome_print_dialog_get_copies (gpd, &copies, &collate);
-		print_config = gnome_print_config_default ();
-		printer = gnome_print_dialog_get_printer (gpd);
-		range = gnome_print_dialog_get_range_page (gpd, &first, &end);
-		gnome_dialog_close (GNOME_DIALOG (gpd));
+		gnome_print_dialog_get_copies (GNOME_PRINT_DIALOG (gnome_print_dialog), 
+					       &copies, &collate);
+		range = gnome_print_dialog_get_range_page (
+			GNOME_PRINT_DIALOG (gnome_print_dialog), &first, &end);
 
 		switch (range) {
 		case GNOME_PRINT_RANGE_CURRENT:
@@ -1144,22 +1176,19 @@ sheet_print (WorkbookControlGUI *wbcg, Sheet *sheet,
 			default_range = PRINT_SHEET_RANGE;
   			break;
   		}
+		gtk_widget_destroy (gnome_print_dialog);
   	}
-	pj = print_job_info_get (sheet, default_range, preview);
-	pj->sorted_print = FALSE;
+
 	if (default_range == PRINT_SHEET_RANGE) {
 		pj->start_page = first-1;
 		pj->end_page = end-1;
 	}
 
-	gnome_print_master_set_paper (gpm, pj->pi->paper);
-	if (printer)
-		gnome_print_master_set_printer (gpm, printer);
-	gnome_print_config_set_int (print_config, GNOME_PRINT_KEY_NUM_COPIES, copies);
-	gnome_print_config_set_boolean (print_config, GNOME_PRINT_KEY_COLLATE, collate);
-
+	print_job_info_save_one_time_defaults (pj);
+	
 	gpm = gnome_print_master_new_from_config (print_config);
 	pj->print_context = gnome_print_master_get_context (gpm);
+	pj->range = default_range;
 
 	/* perform actual printing */
 	switch (pj->range) {
@@ -1185,12 +1214,9 @@ sheet_print (WorkbookControlGUI *wbcg, Sheet *sheet,
 
 	gnome_print_master_close (gpm);
 
-	if (preview) {
-		gboolean landscape = pj->pi->orientation == PRINT_ORIENT_HORIZONTAL;
-		pmp = gnome_print_master_preview_new_with_orientation
-			(gpm, _("Print preview"), landscape);
-		gtk_widget_show (GTK_WIDGET (pmp));
-	} else {
+	if (preview)
+		gtk_widget_show ( gnome_print_master_preview_new (gpm, _("Print preview")));
+	else {
 		int result = gnome_print_master_print (gpm);
 		if (result == -1) {
 			/*
@@ -1201,7 +1227,7 @@ sheet_print (WorkbookControlGUI *wbcg, Sheet *sheet,
 					 _("Printing failed"));
 		}
 	}
-	gtk_object_unref (GTK_OBJECT (gpm));
+
+	g_object_unref (G_OBJECT (gpm));
   	print_job_info_destroy (pj);
-#endif
 }
