@@ -31,7 +31,7 @@
 #include <bonobo/bonobo-item-container.h>
 #include <bonobo/bonobo-stream-memory.h>
 
-#include <gal/util/e-util.h>
+#include <gsf/gsf-impl-utils.h>
 
 #ifdef GNOME2_CONVERSION_COMPLETE
 static SheetObjectClass *sheet_object_bonobo_parent_class;
@@ -325,8 +325,8 @@ sheet_object_bonobo_class_init (GtkObjectClass *object_class)
 	sheet_object_class->write_xml	  = sheet_object_bonobo_write_xml;
 }
 
-E_MAKE_TYPE (sheet_object_bonobo, "SheetObjectBonobo", SheetObjectBonobo,
-	     sheet_object_bonobo_class_init, NULL, SHEET_OBJECT_TYPE);
+GSF_CLASS (SheetObjectBonobo, sheet_object_bonobo,
+	   sheet_object_bonobo_class_init, NULL, SHEET_OBJECT_TYPE);
 
 SheetObjectBonobo *
 sheet_object_bonobo_construct (SheetObjectBonobo   *sob,
