@@ -12,12 +12,14 @@
 #include "sheet-object-impl.h"
 #include "sheet-control-gui.h"
 #include "gnumeric-canvas.h"
+#include "gnumeric-pane.h"
 
-#include <gal/util/e-util.h>
+#include <gsf/gsf-impl-utils.h>
 #include <libgnomecanvas/gnome-canvas-pixbuf.h>
 #include <libgnomecanvas/gnome-canvas-rect-ellipse.h>
 
 #include <math.h>
+#include <string.h>
 
 struct _SheetObjectImage {
 	SheetObject  sheet_object;
@@ -68,6 +70,7 @@ sheet_object_image_new (char const   *type,
 	} else
 		soi->data = data;
 
+	soi->sheet_object.anchor.direction = SO_DIR_DOWN_RIGHT;
 	return SHEET_OBJECT (soi);
 }
 
@@ -181,46 +184,48 @@ soi_get_pixbuf (SheetObjectImage *soi, double scale)
 }
 
 static GObject *
-sheet_object_image_new_view (SheetObject *so, SheetControlGUI *scg)
+sheet_object_image_new_view (SheetObject *so, SheetControl *sc, gpointer key)
 {
-	/* FIXME : this is bogus */
-	GnumericCanvas *gcanvas = scg_pane (scg, 0);
+	GnumericCanvas *gcanvas = ((GnumericPane *)key)->gcanvas;
 	SheetObjectImage *soi = SHEET_OBJECT_IMAGE (so);
 	GnomeCanvasItem *item = NULL;
 	GdkPixbuf	*pixbuf;
 
 	g_return_val_if_fail (IS_SHEET_OBJECT_IMAGE (so), NULL);
-	g_return_val_if_fail (IS_SHEET_CONTROL_GUI (scg), NULL);
+	g_return_val_if_fail (IS_SHEET_CONTROL (sc), NULL);
 
+	gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (gcanvas->sheet_object_group));
 	pixbuf = soi_get_pixbuf (soi, 1.);
 	if (pixbuf != NULL)
-		item = gnome_canvas_item_new (gcanvas->object_group,
+		item = gnome_canvas_item_new (gcanvas->sheet_object_group,
 			GNOME_TYPE_CANVAS_PIXBUF,
 			"pixbuf", pixbuf,
 			NULL);
 	else
-		item = gnome_canvas_item_new (gcanvas->object_group,
+		item = gnome_canvas_item_new (gcanvas->sheet_object_group,
 			GNOME_TYPE_CANVAS_RECT,
 			"fill_color",		"white",
 			"outline_color",	"black",
 			"width_units",		1.,
 			NULL);
 
-	scg_object_register (so, item);
+	gnm_pane_object_register (so, item);
 	return G_OBJECT (item);
 }
 
 static void
-sheet_object_image_update_bounds (SheetObject *so, GObject *view,
-				  SheetControlGUI *scg)
+sheet_object_image_update_bounds (SheetObject *so, GObject *view_obj)
 {
 	double coords [4];
+	GnomeCanvasItem   *view = GNOME_CANVAS_ITEM (view_obj);
+	SheetControlGUI	  *scg  =
+		SHEET_CONTROL_GUI (sheet_object_view_control (view_obj));
 
 	scg_object_view_position (scg, so, coords);
 
 	/* handle place holders */
 	if (GNOME_IS_CANVAS_PIXBUF (view))
-		gnome_canvas_item_set (GNOME_CANVAS_ITEM (view),
+		gnome_canvas_item_set (view,
 			"x",	  MIN (coords [0], coords [2]),
 			"y",	  MIN (coords [1], coords [3]),
 			"width",  fabs (coords [2] - coords [0]),
@@ -229,7 +234,7 @@ sheet_object_image_update_bounds (SheetObject *so, GObject *view,
 			"height_set", TRUE,
 			NULL);
 	else
-		gnome_canvas_item_set (GNOME_CANVAS_ITEM (view),
+		gnome_canvas_item_set (view,
 			"x1", MIN (coords [0], coords [2]),
 			"y1", MIN (coords [1], coords [3]),
 			"x2", MAX (coords [0], coords [2]),
@@ -237,9 +242,9 @@ sheet_object_image_update_bounds (SheetObject *so, GObject *view,
 			NULL);
 
 	if (so->is_visible)
-		gnome_canvas_item_show (GNOME_CANVAS_ITEM (view));
+		gnome_canvas_item_show (view);
 	else
-		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (view));
+		gnome_canvas_item_hide (view);
 }
 
 static gboolean
@@ -314,6 +319,15 @@ sheet_object_image_print (SheetObject const *so, GnomePrintContext *ctx,
 	gnome_print_grestore (ctx);
 }
 
+static void
+sheet_object_image_default_size (SheetObject const *so, double *w, double *h)
+{
+	GdkPixbuf *buf = soi_get_pixbuf (SHEET_OBJECT_IMAGE (so), 1.);
+	*w = gdk_pixbuf_get_width  (buf);
+	*h = gdk_pixbuf_get_height (buf);
+	g_object_unref (buf);
+}
+
 static void
 sheet_object_image_class_init (GObjectClass *object_class)
 {
@@ -334,6 +348,7 @@ sheet_object_image_class_init (GObjectClass *object_class)
 	sheet_object_class->user_config   = NULL;
 	sheet_object_class->print         = sheet_object_image_print;
 	sheet_object_class->rubber_band_directly = TRUE;
+	sheet_object_class->default_size  = sheet_object_image_default_size;
 }
 
 static void
@@ -351,6 +366,6 @@ sheet_object_image_init (GObject *obj)
 	so->anchor.direction = SO_DIR_NONE_MASK;
 }
 
-E_MAKE_TYPE (sheet_object_image, "SheetObjectImage", SheetObjectImage,
-	     sheet_object_image_class_init, sheet_object_image_init,
-	     SHEET_OBJECT_TYPE);
+GSF_CLASS (SheetObjectImage, sheet_object_image,
+	   sheet_object_image_class_init, sheet_object_image_init,
+	   SHEET_OBJECT_TYPE);
