@@ -4,6 +4,7 @@
 
 #include "gnumeric.h"
 #include "colrow.h"
+#include "sheet-view.h"
 
 typedef GList ColStyleList;
 typedef struct _SheetPrivate SheetPrivate;
@@ -13,7 +14,7 @@ struct _Sheet {
 	int         index_in_wb;
 	Workbook    *workbook;
 
-	GList       *s_controls;
+	GPtrArray   *sheet_views;
 
 	char        *name_quoted;
 	char        *name_unquoted;
@@ -24,28 +25,6 @@ struct _Sheet {
 
 	GHashTable  *cell_hash;	/* The cells in hashed format */
 
-	CellPos	 edit_pos;	/* Cell that would be edited */
-	CellPos	 edit_pos_real;	/* Even in the middle of a merged cell */
-
-	struct {
-		/* Static corner to rubber band the selection range around */
-		CellPos	 base_corner;
-		/* Corner that is moved when the selection range is extended */
-		CellPos	 move_corner;
-	} cursor;
-
-	/*
-	 * an ordered list of SheetSelections, the first of
-	 * which corresponds to the range base_corner:move_corner
-	 */
-	GList       *selections;
-
-	/*
-	 * Same as the above, contains the currently anted regions
-	 * on the sheet
-	 */
-	GList       *ants;
-
 	/* User defined names */
 	GList      *names;
 
@@ -69,8 +48,12 @@ struct _Sheet {
 	gboolean    outline_symbols_below;
 	gboolean    outline_symbols_right;
 
+	gboolean    has_hidden_rows;  /* Filtered or hidden rows */
+
         SolverParameters *solver_parameters;
 
+	gint simulation_round;
+
 	GnmDepContainer *deps;
 
 	GSList		 *list_merged;
@@ -79,10 +62,6 @@ struct _Sheet {
 	PrintInformation *print_info;
 	StyleColor	 *tab_color;
 	StyleColor	 *tab_text_color;
-
-	CellPos initial_top_left;	/* belongs in sheetView */
-	CellPos frozen_top_left;	/* these may also belong there */
-	CellPos unfrozen_top_left;
 };
 
 #define SHEET_SIGNATURE 0x12349876
@@ -93,26 +72,11 @@ Sheet      *sheet_dup			(Sheet const *source_sheet);
 void        sheet_destroy		(Sheet *sheet);
 void        sheet_destroy_contents	(Sheet *sheet);
 void        sheet_rename		(Sheet *sheet, char const *new_name);
-void	    sheet_set_initial_top_left	(Sheet *sheet, int col, int row);
-void	    sheet_freeze_panes		(Sheet *sheet,
-					 CellPos const *frozen_top_left,
-					 CellPos const *unfrozen_top_left);
-gboolean    sheet_is_frozen		(Sheet const *sheet);
-void	    sheet_set_tab_color		(Sheet *sheet, StyleColor *tab_color, 
+void	    sheet_set_tab_color		(Sheet *sheet, StyleColor *tab_color,
 					 StyleColor *text_color);
 
 void        sheet_set_zoom_factor	(Sheet *sheet, double factor,
 					 gboolean force, gboolean respan);
-void        sheet_cursor_set		(Sheet *sheet,
-					 int edit_col, int edit_row,
-					 int base_col, int base_row,
-					 int move_col, int move_row,
-					 Range const *cursor_bound);
-void        sheet_set_edit_pos		(Sheet *sheet, int col, int row);
-
-/* deprecated : this will be removed when sheetViews are added */
-void        sheet_make_cell_visible	(Sheet *sheet, int col, int row,
-					 gboolean couple_panes);
 
 /* Cell management */
 Cell       *sheet_cell_get		(Sheet const *sheet, int col, int row);
@@ -120,13 +84,11 @@ Cell       *sheet_cell_fetch		(Sheet *sheet, int col, int row);
 Cell       *sheet_cell_new		(Sheet *sheet, int col, int row);
 void        sheet_cell_remove		(Sheet *sheet, Cell *cell, gboolean redraw);
 
-/* Iteration utilities */
-/* See also : workbook_foreach_cell_in_range */
-Value      *sheet_foreach_cell_in_range	(Sheet *sheet, gboolean only_existing,
+Value      *sheet_foreach_cell_in_range	(Sheet *sheet, CellIterFlags flags,
 					 int start_col, int start_row,
 					 int end_col, int end_row,
-					 ForeachCellCB callback,
-					 void *closure);
+					 CellIterFunc callback,
+					 gpointer     closure);
 GPtrArray  *sheet_cells                  (Sheet *sheet,
 					  int start_col, int start_row,
 					  int end_col, int end_row,
@@ -234,6 +196,7 @@ gboolean sheet_ranges_split_region   (Sheet const *sheet,
 				      WorkbookControl *wbc, char const *cmd);
 gboolean sheet_range_contains_region (Sheet const *sheet, Range const *r,
 				      WorkbookControl *wbc, char const *cmd);
+void	 sheet_range_bounding_box    (Sheet const *sheet, Range *r);
 
 /* Redraw */
 void        sheet_redraw_all              (Sheet const *sheet, gboolean header);
@@ -242,9 +205,6 @@ void        sheet_redraw_range            (Sheet const *sheet, Range const *r);
 void        sheet_redraw_region      	  (Sheet const *sheet,
 				           int start_col, int start_row,
 				           int end_col,   int end_row);
-void	    sheet_redraw_headers          (Sheet const *sheet,
-					   gboolean col, gboolean row,
-					   Range const* r /* optional == NULL */);
 
 void        sheet_unant                    (Sheet *sheet);
 void        sheet_ant                      (Sheet *sheet, GList *ranges);
@@ -259,8 +219,6 @@ void        sheet_update                   (Sheet const *s);
 void	    sheet_scrollbar_config	   (Sheet const *s);
 void        sheet_adjust_preferences   	   (Sheet const *s,
 					    gboolean redraw, gboolean resize);
-void        sheet_menu_state_enable_insert (Sheet *s,
-					    gboolean col, gboolean row);
 
 void        sheet_set_dirty               (Sheet *sheet, gboolean is_dirty);
 gboolean    sheet_is_pristine             (Sheet const *sheet);
@@ -326,22 +284,33 @@ void  sheet_clear_region (WorkbookControl *context,
 			  int end_col, int end_row,
 			  int clear_flags);
 
-void	sheet_attach_control (Sheet *sheet, SheetControl *sc);
-void    sheet_detach_control (SheetControl *sc);
+void	sheet_attach_view (Sheet *sheet, SheetView *sv);
+void    sheet_detach_view (SheetView *sv);
+SheetView *sheet_get_view (Sheet const *sheet, WorkbookView const *wbv);
 
-#define SHEET_FOREACH_CONTROL(sheet, control, code)			\
-do {									\
-	GList *PtR, *NexTPtR;						\
-	for (PtR = (sheet)->s_controls; PtR != NULL ; PtR = NexTPtR) {	\
-		SheetControl *control = PtR->data;			\
-		NexTPtR = PtR->next;					\
-		code							\
-	}								\
+#define SHEET_FOREACH_VIEW(sheet, view, code)					\
+do {										\
+	int InD;								\
+	GPtrArray *views = (sheet)->sheet_views;				\
+	if (views != NULL) /* Reverse is important during destruction */	\
+		for (InD = views->len; InD-- > 0; ) {				\
+			SheetView *view = g_ptr_array_index (views, InD);	\
+			code							\
+		}								\
 } while (0)
 
+#define SHEET_FOREACH_CONTROL(sheet, view, control, code)		\
+	SHEET_FOREACH_VIEW((sheet), view, 				\
+		SHEET_VIEW_FOREACH_CONTROL(view, control, code);)
 
+/*
+ * Walk the dependents.  WARNING: Note, that it is only valid to muck with
+ * the current dependency in the code.
+ */
 #define SHEET_FOREACH_DEPENDENT(sheet, dep, code)					\
   do {											\
+	/* Maybe external deps here.  */						\
+											\
 	if ((sheet)->deps) {								\
 		DEPENDENT_CONTAINER_FOREACH_DEPENDENT ((sheet)->deps, dep, code);	\
 	}										\
