@@ -20,6 +20,7 @@ struct _WorkbookControlGUI {
 	GtkNotebook *notebook;
 
 #ifdef WITH_BONOBO
+	GHashTable *custom_ui_components;
 	BonoboUIComponent *uic;
 	GtkWidget *progress_bar;
 #else
@@ -57,6 +58,8 @@ struct _WorkbookControlGUI {
 	GtkWidget  *menu_item_sheet_outline_symbols_below;
 	GtkWidget  *menu_item_sheet_outline_symbols_right;
 
+        GtkWidget  *menu_item_filter_show_all;
+
 	/* Toolbars */
 	GtkWidget *standard_toolbar;
 	GtkWidget *format_toolbar;
